rm(list=ls())
###############################################
####
#### Sript produce Annotation files. 
####  There are two annotation files SNP_annot and CN_annot for Affymetrix SNP6.0 
####
####   Each annotation file would contain 6 columns:
####  SNP annotation columns:    SNP_ID, Index, Chr_N(1-24), Chr_Arm(1,2), Position, Nearest_CN_Index 
####  CN annotation columns:      CN_ID, Index, Chr_N(1-24), Chr_Arm(1,2), Position, Nearest_SNP_Index 
####
####  We will produce Annotations from one of the exports
####
###############################################


###############################################
### File paths and other parameters

### !!!!! Put the PATH to your data  !!!!

pPath<-"C:/GAP_AffymetrixSNP6_workflow_homepage/"

inputPath<-paste(pPath,"Exports/",sep="")
outputPath<-paste(pPath,"",sep="")



### !!!!! Correct the "Version" of Affymetrix annotation files, for example, "na31"  !!!!
SNP_annot_fileName<-"SNP_annot_Affy6_Version.csv"
CN_annot_fileName<-"CN_annot_Affy6_Version.csv"


###############################################
### Chromosome arm information
#### !!!!!!!  choose correct genome version:  cytoBand_adapted_fr_hg18.csv

cyto_Band<-read.csv(paste(pPath,"cytoBand_adapted_fr_hg18.csv",sep=""),header=T,fill=T)
cyto_Band<-cyto_Band[order(cyto_Band$Chr_N,cyto_Band$Chr_arm,decreasing=F),]
cytoArm<-cyto_Band
for(k in 2:dim(cyto_Band)[[1]]){
if((cyto_Band$Chr_arm[k-1]==1)&&(cyto_Band$Chr_arm[k]==2)){cytoArm$Index[k-1]<-1
}else{cytoArm$Index[k]<-0}
}
cytoArm$Index[1]<-0
tt<-which(cytoArm$Index==0);if(length(tt)>0){cytoArm<-cytoArm[-tt,]}
tt<-which(cytoArm$Chr_arm==1);if(length(tt)>0){cytoArm<-cytoArm[-tt,]}

cytoArm<-cbind(cytoArm$Chr_N,cytoArm$Start)
colnames(cytoArm)<-c("Chr_N","Start_q")

###############################################


########### Reading your exports


### !!!!! Put your PATTERN<-"XXX", where XXX is some particular string from file names !!!!
### !!!!! Leave PATTERN="", if you process all files from Exports folder !!!!

PATTERN<-""
segFiles <- list.files(inputPath,pattern=PATTERN, full.names=T) 





fileNames <- gsub(inputPath,"",segFiles) 
fileNames <- gsub(".txt","",fileNames) 

fileNames

nSample<-length(fileNames)

###############################################
### Reading the first available export file 

ii<-1

print(ii)

### !!!!! Look at your export files !!!!!!

###  !!! ID_column is the column with SNP IDs (need to be unique)
### !!!  correct the constant acording to your export file

ID_column<-1


aa<-scan(segFiles[ii],nlines=1,what="character",sep="\t")
k<-1
while((length(grep("#",aa))!=0)&&(grep("#",aa)==1)){
aa<-scan(segFiles[ii],skip=k,nlines=1,what="character",sep="\t")
k<-k+1
}

dataF<-read.delim(segFiles[ii],skip=k-1,header=T,fill=T,na.strings="null",row.names=ID_column)

colnames(dataF)
rownames(dataF)[1]

c_chr<-1
c_pos<-2


#################    !!!!!!!!!!!     ###################
###########
########### columns of dataF matrix should be adjusted to export file
###########  column c_chr=1 : chromosome 
###########  column c_pos=2 : position
###########   
########### If you have other column order, please, put their corresponding 


nSNP<-dim(dataF)[[1]]
Chr<-as.character(dataF[,c_chr])

tt<-which(Chr=="X");if(length(tt)>0){Chr[tt]<-23}
tt<-which(Chr=="Y");if(length(tt)>0){Chr[tt]<-24}
tt<-which(Chr=="M");if(length(tt)>0){Chr[tt]<-25}
Chr<-as.numeric(Chr)

rNames<-as.character(rownames(dataF))
dataF<-cbind(seq(1,nSNP,1),Chr,Chr,dataF[,c_pos])
dataF<-data.matrix(dataF)
colnames(dataF)<-c("Index","Chr_N","Chr_arm","Position")
rownames(dataF)<-rNames

C_Ind<-1
C_Chr<-2
C_Arm<-3
C_Pos<-4


dataF[,C_Arm]<-1

chr<-1
for(chr in 1:dim(cytoArm)[[1]]){
	tt<-which(dataF[,C_Chr]==chr)
	if(length(tt)>0){
		qq<-which(dataF[tt,C_Pos]>=cytoArm[chr,2])
		if(length(qq)>0){dataF[tt[qq],C_Arm]<-2}
	}
}#chr


table(dataF[,C_Chr],dataF[,C_Arm])

rNames<-as.character(rownames(dataF))
tt<-which(grepl("CN",rNames)==TRUE)

CN_annot<-dataF[tt,]
SNP_annot<-dataF[-tt,]

CN_annot<-CN_annot[order(CN_annot[,C_Chr],CN_annot[,C_Pos],decreasing=F),]
CN_annot[,1]<-seq(1,dim(CN_annot)[[1]])

SNP_annot<-SNP_annot[order(SNP_annot[,C_Chr],SNP_annot[,C_Pos],decreasing=F),]
SNP_annot[,1]<-seq(1,dim(SNP_annot)[[1]])


############### If saving SNP_annot and CN_annot

#dataFName<-paste(outputPath,"SNP_annotation_an31.txt",sep="")
#write.table(SNP_annot, file = dataFName,quote=F,col.names=F,row.names=F)

#dataFName<-paste(outputPath,"CN_annotation_an31.txt",sep="")
#write.table(CN_annot, file = dataFName,quote=F,col.names=F,row.names=F)

################ Adjusting SNP and CN annotation

CN_annot<-data.matrix(CN_annot)
SNP_annot<-data.matrix(SNP_annot)

C_Ind<-1
C_Chr<-2
C_Arm<-3
C_Pos<-4

CHR<-as.numeric(table(SNP_annot[,C_Arm],SNP_annot[,C_Chr]))
CHR_snp<-c(0,CHR)

CHR<-as.numeric(table(CN_annot[,C_Arm],CN_annot[,C_Chr]))
CHR_cn<-c(0,CHR)

for(k in 2:length(CHR_snp)){
CHR_snp[k]<-CHR_snp[k]+CHR_snp[k-1]
}
for(k in 2:length(CHR_cn)){
CHR_cn[k]<-CHR_cn[k]+CHR_cn[k-1]
}


############### Add SNP position to CN annotation

CN_annot<-cbind(CN_annot,CN_annot[,C_Pos])
CN_annot[,C_Pos+1]<-0
colnames(CN_annot)[C_Pos+1]<-"nearest_SNP_Index"
CN_annot<-data.matrix(CN_annot)

k<-2
for(k in 2:length(CHR_cn)){
print(k)

if(CHR_cn[k]-(CHR_cn[k-1]+1)>3){

tts<-CN_annot[(CHR_cn[k-1]+1):CHR_cn[k],C_Pos]
tts<-cbind(tts,0,0,0,-seq(CHR_cn[k-1]+1,CHR_cn[k]))

if(CHR_snp[k]-(CHR_snp[k-1]+1)>3){

ttc<-SNP_annot[(CHR_snp[k-1]+1):CHR_snp[k],C_Pos]
ttc<-cbind(ttc,1,1,1,seq(CHR_snp[k-1]+1,CHR_snp[k]))
tt<-rbind(ttc,tts)
tt<-tt[order(tt[,1]),]
nn<-dim(tt)[[1]]

tt[1:(nn-1),2]<-(tt[2:nn,1]-tt[1:(nn-1),1])*tt[2:nn,4]
tt[2:nn,3]<-(tt[2:nn,1]-tt[1:(nn-1),1])*tt[1:(nn-1),4]

kk<-which(tt[,2]>40000)
tt[kk,2]<-0

kk<-which(tt[,3]>40000)
tt[kk,3]<-0

kk1<-which((tt[,2]-tt[,3])*(1-sign(tt[,4]))*sign(tt[,2])<0)
kk2<-which((tt[,3]-tt[,2])*(1-sign(tt[,4]))*sign(tt[,3])<0)
tt[kk1,5]<-tt[kk1+1,5]
tt[kk2,5]<-tt[kk2-1,5]
tt[kk1,2]<-0
tt[kk2,3]<-0
tt[kk1,3]<-0
tt[kk2,2]<-0

kk1<-which(tt[,3]*(1-sign(tt[,4]))>0)
kk2<-which(tt[,2]*(1-sign(tt[,4]))>0)
tt[kk1,5]<-tt[kk1-1,5]
tt[kk2,5]<-tt[kk2+1,5]

tt[1:(nn-1),2]<-(tt[2:nn,1]-tt[1:(nn-1),1])*tt[2:nn,4]
tt[2:nn,3]<-(tt[2:nn,1]-tt[1:(nn-1),1])*tt[1:(nn-1),4]

kk<-which(tt[,4]==0)
tt<-tt[kk,]


kk<-table(tt[,5])
kk1<-which(kk>1)
kk<-kk[kk1]

p<-2
for(p in 1:length(kk)){
	kk1<-as.numeric(rownames(kk)[p])
	pp<-which(tt[,5]==kk1)
	pp1<-which.min(abs(tt[pp,1]-ttc[which(ttc[,5]==kk1),1]))
	tt[pp,5]<-(tts[pp,5])
	tt[pp[pp1],5]<-kk1
}

kk<-which(tt[,5]<0)
tt[kk,5]<-0
CN_annot[(CHR_cn[k-1]+1):CHR_cn[k],C_Pos+1]<-tt[,C_Pos+1]

}
}
}#CHR

table(sign(CN_annot[,C_Pos]-CN_annot[,C_Pos+1]))
table(sign(CN_annot[,C_Pos]+CN_annot[,C_Pos+1]))


fName<-paste(pPath,CN_annot_fileName,sep="")
write.table(CN_annot, file = fName,quote=F,col.names=NA,row.names=T,sep=",")


############### Add CN position to SNP annotation

SNP_annot<-cbind(SNP_annot,SNP_annot[,C_Pos])
SNP_annot[,C_Pos+1]<-0
colnames(SNP_annot)[C_Pos+1]<-"Nearest_CN_Index"
SNP_annot<-data.matrix(SNP_annot)

k<-2
for(k in 2:length(CHR_snp)){
print(k)

if(CHR_snp[k]-(CHR_snp[k-1]+1)>3){

tts<-SNP_annot[(CHR_snp[k-1]+1):CHR_snp[k],C_Pos]
tts<-cbind(tts,0,0,0,-seq(CHR_snp[k-1]+1,CHR_snp[k]))

if(CHR_cn[k]-(CHR_cn[k-1]+1)>3){

ttc<-CN_annot[(CHR_cn[k-1]+1):CHR_cn[k],C_Pos]
ttc<-cbind(ttc,1,1,1,seq(CHR_cn[k-1]+1,CHR_cn[k]))
tt<-rbind(ttc,tts)
tt<-tt[order(tt[,1]),]
nn<-dim(tt)[[1]]

tt[1:(nn-1),2]<-(tt[2:nn,1]-tt[1:(nn-1),1])*tt[2:nn,4]
tt[2:nn,3]<-(tt[2:nn,1]-tt[1:(nn-1),1])*tt[1:(nn-1),4]

kk<-which(tt[,2]>40000)
tt[kk,2]<-0

kk<-which(tt[,3]>40000)
tt[kk,3]<-0

kk1<-which((tt[,2]-tt[,3])*(1-sign(tt[,4]))*sign(tt[,2])<0)
kk2<-which((tt[,3]-tt[,2])*(1-sign(tt[,4]))*sign(tt[,3])<0)
tt[kk1,5]<-tt[kk1+1,5]
tt[kk2,5]<-tt[kk2-1,5]
tt[kk1,2]<-0
tt[kk2,3]<-0
tt[kk1,3]<-0
tt[kk2,2]<-0

kk1<-which(tt[,3]*(1-sign(tt[,4]))>0)
kk2<-which(tt[,2]*(1-sign(tt[,4]))>0)
tt[kk1,5]<-tt[kk1-1,5]
tt[kk2,5]<-tt[kk2+1,5]

tt[1:(nn-1),2]<-(tt[2:nn,1]-tt[1:(nn-1),1])*tt[2:nn,4]
tt[2:nn,3]<-(tt[2:nn,1]-tt[1:(nn-1),1])*tt[1:(nn-1),4]

kk<-which(tt[,4]==0)
tt<-tt[kk,]


kk<-table(tt[,5])
kk1<-which(kk>1)
kk<-kk[kk1]

p<-2
for(p in 1:length(kk)){
	kk1<-as.numeric(rownames(kk)[p])
	pp<-which(tt[,5]==kk1)
	pp1<-which.min(abs(tt[pp,1]-ttc[which(ttc[,5]==kk1),1]))
	tt[pp,5]<-(tts[pp,5])
	tt[pp[pp1],5]<-kk1
}

kk<-which(tt[,5]<0)
tt[kk,5]<-0
SNP_annot[(CHR_snp[k-1]+1):CHR_snp[k],C_Pos+1]<-tt[,C_Pos+1]

}
}
}#CHR

table(sign(SNP_annot[,C_Pos]-SNP_annot[,C_Pos+1]))
table(sign(SNP_annot[,C_Pos]+SNP_annot[,C_Pos+1]))




fName<-paste(pPath,SNP_annot_fileName,sep="")
write.table(SNP_annot, file = fName,quote=F,col.names=NA,row.names=T,sep=",")


########## END  ######################






