##################
#####
#####    This script provides the preparation workflow for copy number and genotype recognition
#####      in Affy 6.0 SNP array profiles with the GAP method (Genome Biology, 2009, Popova et al)
#####      namesly, 
#####     1) segmentation of CN(copy number) and AD (allelic differences) profiles by CBS
#####     2) construction of segmented summary file for further analysis with GAP method
#####
#####    As an input we use vector forms of (1)Log ratio and (2)Allelic difference from SNP probes, 
#####        and (3) Copy Number probes. All vectors are obtained directly from Genotyping Console. 
#####    As vector annotation we use Affy6_SNP_short_annotCN.csv file   
#####
#####
#####    The input vectors from Vectors_and_Segmentations folder: V_LRR_Sample1,V_AD_Sample1,V_CN_Sample1
#####   
#####    The output segmented summary files : segm_Sample1  are saved in Vectors_and_Segmentations folder
#####     
#####
#####    The input vector: V_AD_Sample1 was modified by adding (-) to homozygous SNPs
#####    
#####
##################

rm(list=ls())

###############################################
### File paths and other parameters

### !!!!! Put the PATH to your data  !!!!

pPath<-"C:/GAP_AffymetrixSNP6_workflow_homepage/"

inputPath<-paste(pPath,"Vectors_and_segmentations/",sep="")
outputPath<-paste(pPath,"Vectors_and_segmentations/",sep="")


### Annotation files
### !!!!! Correct the names of annotation files according to script_0  !!!!

CN_annot<-read.csv(paste(pPath,"CN_annot_Affy6_Version.csv",sep=""),header=T,fill=T,stringsAsFactors = F,row.names=1)
SNP_annot<-read.csv(paste(pPath,"SNP_annot_Affy6_Version.csv",sep=""),header=T,fill=T,stringsAsFactors = F,row.names=1)
CN_annot<-data.matrix(CN_annot)
SNP_annot<-data.matrix(SNP_annot)
C_Ind<-1;C_Chr<-2;C_Pos<-4


### Both source files need SNP_annot file defined above

source(paste(pPath,"script_functions_GAP_construction_CBS_Affy6.R",sep=""))
source(paste(pPath,"script_functions_GAP_recognition_Affy6.R",sep=""))


#### !!!!  All V_LRR_Sample1 files form Vectors_and_Segmentations folder will be processed  !!!!

segFiles <- list.files(inputPath, pattern=vect_LR_pattern,full.names=T) 

fileNames <- gsub(inputPath,"",segFiles) 
fileNames <- gsub(vect_LR_pattern,"",fileNames) 
fileNames <- gsub(".txt","",fileNames) 
fileNames
nSample<-length(segFiles)


##############  MAIN CYCLE

ii<-1

for(ii in 1:nSample){

print(paste("processing  ",fileNames[ii]))

## may take 1-5 min per sample to calculate the break points by CBS algorithm and segmentation summary 

tmp<-CBS_breakpoints_and_GAPconstruction(ii)

plot(tmp[,c_adL],tmp[,c_cn],cex=tmp[,c_len]/quantile(tmp[,c_len],prob=0.9),main=paste("GAP ",fileNames[ii]),
                xlab="Allelic Difference",ylab="Log ratio")
abline(v=0)
abline(-1,1)

fName<-paste(outputPath,vect_Segm_pattern,fileNames[ii],".txt",sep="")
write.table(tmp, file = fName,quote=F,row.names=F,sep="\t")

}#ii

########### END OF MAIN CYCLE