rm(list=ls())

#############################################################################
#######
#######		 This script provides the preliminary GAP Pattern recognition
#######		 It is useful to check the quality of automatic recognition pattern 
#######                and correct it manually if it is possible.
#######            If you do not see any clear pattern, it means that the sample is
#######               very contaminated or of a poor quality.
#######                You may decide to exclude this sample.
#######
#######            As a result you will see GAP plots in GAP_plots folder and
#######                  GAP parameters file in Vectors_and_Segmentations folder.
#######
#############################################################################

### !!!!! Put the PATH to your data  !!!!

pPath<-"C:/GAP_AffymetrixSNP6_workflow_homepage/"

inputPath<-paste(pPath,"Vectors_and_Segmentations/",sep="")
outputPath<-paste(pPath,"Vectors_and_Segmentations/",sep="")
outputPath_Plot<-paste(pPath,"Plots_GAP/",sep="")


### Annotation files
### !!!!! Correct the names of annotation files according to script_0  !!!!

CN_annot<-read.csv(paste(pPath,"CN_annot_Affy6_Version.csv",sep=""),header=T,fill=T,stringsAsFactors = F,row.names=1)
SNP_annot<-read.csv(paste(pPath,"SNP_annot_Affy6_Version.csv",sep=""),header=T,fill=T,stringsAsFactors = F,row.names=1)
CN_annot<-data.matrix(CN_annot)
SNP_annot<-data.matrix(SNP_annot)
C_Ind<-1;C_Chr<-2;C_Pos<-4


### Both source files need SNP_annot file defined above

source(paste(pPath,"script_functions_GAP_construction_CBS_Affy6.R",sep=""))
source(paste(pPath,"script_functions_GAP_recognition_Affy6.R",sep=""))


### !!!!! Give your favorit name to GAP_parameters_file_name, if you want   !!!!
GAP_parameters_file_name<-"GAP_parameters_Example"


### !!!!! All segm_Sample1 files from Vectors_and_Segmentations folder will be processed !!!!

segFiles <- list.files(inputPath, pattern=vect_Segm_pattern,full.names=T) 

fileNames <- gsub(inputPath,"",segFiles) 
fileNames <- gsub(vect_Segm_pattern,"",fileNames) 
fileNames <- gsub(".txt","",fileNames) 

fileNames

nSample<-length(segFiles)

matrCoef<-matrix(0,nSample,4)
matrCoef[,1]<-seq(1,nSample)
rownames(matrCoef)<-c(1:nSample)
colnames(matrCoef)<-c("num_Sample","p_BAF","q_LRR","2copy_LRR")


################## MAIN CYCLE

ii<-1

for(ii in 1:nSample){

	print(paste("processing  ",fileNames[ii]))
	tmp<-read.delim(segFiles[ii],header=T,sep="\t")
	tmp<-data.matrix(tmp)

	notNAs<-which(!is.na(tmp[,c_cn]+tmp[,c_adL]))

### fitting GAP model by exaustive search. May take 1-2 min per sample

	result<-GAP_pattern(tmp[notNAs,c_adL],tmp[notNAs,c_cn],tmp[notNAs,c_len],showGAPTemplate=T,saveGAPplot=T,
		GAPplot_fileName=paste(outputPath_Plot,"GAP_plot_",fileNames[ii],".jpg",sep=""))
	

	matrCoef[ii,2]<-result[1]
	matrCoef[ii,3]<-result[2]
	matrCoef[ii,4]<-result[3]
	rownames(matrCoef)[ii]<-fileNames[ii]

}#ii

################ END MAIN CYCLE

matrCoef<-round(matrCoef,3)

fName<-paste(inputPath,GAP_parameters_file_name,".csv",sep="")
write.table(matrCoef, file = fName,quote=F,col.names=NA,row.names=T,sep=",")

##############  END AUTOMATICAL FITTING









################ MANUAL FITTING  (ALL commands before the MAIN CYCLE should be executed!!!)

fileNames

### !!!!! Chose number of sample for manual fitting  !!!!!!!
     
      ii<-1

	print(paste("processing  ",fileNames[ii]))
	tmp<-read.delim(segFiles[ii],header=T,sep="\t")
	tmp<-data.matrix(tmp)

	notNAs<-which(!is.na(tmp[,c_cn]+tmp[,c_adL]))


########### MANUAL FITTING CYCLE

### !!!!! step 1. Chose the parameters you feel are better than automatically detected ones

p_BAF<-0.7;q_LRR<-0.16;Delta<-c(2,-0.1)

### Delta is vector of 2 values: c(Copy number, LRR of this Copy number)

### !!!!! step 2.  See the results

result<-CopyNumber_and_Genotype_patternWOfitting(tmp[notNAs,c_adL],tmp[notNAs,c_cn],tmp[notNAs,c_len],showGAPTemplate=T,power=3/4,p_BAF,q_LRR,Delta)

### !!!!! Go back to step 1 until you find a good combination.

########## END MANUAL FITTING CYCLE

### !!!!!  step 3.  Open GAP_parameters_file in excel and correct it manually.


