
###
###  FUNCTIONS, PACKAGES and GLOBAL VARIABLES for GAP recogntion fitting
###

#########################################################################
##################
################## Function routhEqual
##################
#########################################################################

routhEqual<-function(x1,x2,rapport,Tolerance){
	if(rapport==0){if(abs(x1-x2)<Tolerance*min(abs(x1),abs(x2))){out<-TRUE}else{out<-FALSE}
	}else{if(abs(x1-x2)<Tolerance*abs(rapport)){out<-TRUE}else{out<-FALSE}}
}



#########################################################################
##################
################## Function LogLike
##################
#########################################################################


logLike<- function(x, Lambda, MU, SD){
    a1 <- (x - MU[1])^2
    if(MU[1]!=0){b1 <- (Lambda/SD[1]) * exp(-a1/2/SD[1]^2)
	}else{b1 <- 2*(Lambda/SD[1]) * exp(-a1/2/SD[1]^2)}
    a2 <- (x - MU[2])^2
    b2 <- ((1 - Lambda)/SD[2]) * exp(-a2/2/SD[2]^2)
    out <- sum(log(b1 + b2))-length(x) * 0.918938533204673
    out
}




#########################################################################
##################
################## Function cut tails of Alleic dif distr
##################
#########################################################################

cutADV<-function(adv,prRight,prLeft,centerBoundary){

tt<-which(is.na(adv))
if((length(tt)!=0)){adv<-adv[-tt]}

if(length(adv)==0){adv<-numeric(0)}else{
tt<-which(adv>quantile(adv, prob=prRight))
if(length(tt)>0){adv<-adv[-tt]}

tt<-length(which(adv<centerBoundary))/length(adv)

if(tt<prLeft){
	tt<-which(adv<quantile(adv, prob=prLeft))
	if(length(tt)>0){adv<-adv[-tt]}
}
}
out<-adv

}
#########################################################################
##################
################## Function Mixture of Two Normals
##################
#########################################################################

normalmixEM2compModif<-function (x, lambda, mu, sigsqrd, eps = 1e-08, maxit = 1000) 
{
    mi1<-c(mu[1],sigsqrd[1],lambda);names(mi1)<-c("mu","D","la")
    mi2<-c(mu[2],sigsqrd[2],1-lambda);names(mi2)<-c("mu","D","la")

    mx <- mean(x)
    const <- length(x) * 0.918938533204673
    dl <- 1 + eps
    iter <- 0
    a1 <- (x - mi1[1])^2
    b1 <- (mi1[3]/sqrt(mi1[2])) * exp(-a1/2/mi1[2])
    a2 <- (x - mi2[1])^2
    b2 <- (mi2[3]/sqrt(mi2[2])) * exp(-a2/2/mi2[2])
    l<-sum(log(b1 + b2))
    ll <- c(mi1,mi2,l)
    postprobs <- b1/(b1 + b2)
    mi1[3] <- mean(postprobs)
    mi2[3]<- (1 - mi1[3])

    while (dl > eps && iter < maxit && mi1[3]*mi2[3]>0) {
        iter <- iter + 1
        mi1[1] <- mean(postprobs * x)/mi1[3]
        mi2[1] <- (mx - mi1[3] * mi1[1])/mi2[3]
        mi1[2] <- mean(postprobs * a1)/mi1[3]
        mi2[2] <- mean((1 - postprobs) * a2)/mi2[3]

        a1 <- (x - mi1[1])^2
        b1 <- (mi1[3]/sqrt(mi1[2])) * exp(-a1/2/mi1[2])
        a2 <- (x - mi2[1])^2
        b2 <- (mi2[3]/sqrt(mi2[2])) * exp(-a2/2/mi2[2])

        oldl <- l
        l <- sum(log(b1 + b2))
		
        dl <- l - oldl
        ll <- cbind(ll,c(mi1,mi2,l))

        postprobs <- b1/(b1 + b2)
        mi1[3] <- mean(postprobs)
	  mi2[3]<- (1 - mi1[3])
    }
  if (mi1[3]==0){ mi2[1] <- mx;mi2[2] <- mean((x - mi2[1])^2)
			l<-logLike(x, Lambda=0, MU=c(mi1[1],mi2[1]), SD=c(sqrt(mi1[2]),sqrt(mi2[2])))
  	      	ll <- cbind(ll,c(mi1,mi2,l+const))
	
  }else{if (mi2[3]==0){ mi1[1] <- mx;mi1[2] <- mean((x - mi1[1])^2)
			l<-logLike(x, Lambda=1, MU=c(mi1[1],mi2[1]), SD=c(sqrt(mi1[2]),sqrt(mi2[2])))
  	      	ll <- cbind(ll,c(mi1,mi2,l+const))
	
		  }
		}

  ll[2,]<-sqrt(ll[2,])
  ll[5,]<-sqrt(ll[5,])
  ll[7,]<-ll[7,]-const 
  out <- ll
  out
}





#########################################################################
##################
################## Function Choose Two Normals Mixture Model
##################
#########################################################################


chooseMix2Model_new<-function(x, lambda, mu, sigsqrd, eps, maxit){

ll<-normalmixEM2compModif(x, lambda, mu, sigsqrd, eps, maxit) 
ll<-ll[,order(ll[7,],decreasing=T)]


if(ll[1,1]<0.35){

	mu2<-ll[4,1]
	sd2<-ll[5,1]
	sd1<-ll[2,1]

	mu<-c(0,0.15,0.2,0.25,0.3)
	lambda<-c(1,0.2,0.4,0.6)
	
	iter<-length(mu)*length(lambda)
	ll<-cbind(matrix(0,7,iter),ll)
	ll[2,1:iter]<-sd1
	ll[4,1:iter]<-mu2
	ll[5,1:iter]<-sd2

	k<-1;i<-1
	for(k in 1:length(mu)){
	for(i in 1:length(lambda)){
		iter<-(k-1)*length(lambda)+i
		ll[1,iter]<-mu[k]
		ll[3,iter]<-lambda[i]
		ll[6,iter]<-1-lambda[i]
		ll[7,iter]<-logLike(x, Lambda=lambda[i], MU=c(mu[k],mu2), SD=c(sd1,sd2))
#normalmixEM2compSDfit(x, lambda=lambda[i], mu=c(mu[k],mu2), sigsqrd=c(sd1,sd2), eps = 1e-01, maxit = 20) 
	}#i
	}#k

ll<-ll[,order(ll[7,],decreasing=T)]

#if close to balance
}else{

}

out<-ll[,1]


}# function



########################################################################
##################
################## Function SD of homoAD band
##################
#########################################################################


getSDofHomozygousADband<-function(tmp, datA, Length,centerBoundary){
k<-1
sdH<-numeric(0)

for(k in 1:dim(tmp)[[1]]){

	if((tmp[k,c_len]>Length)&&(tmp[k,c_adL]<centerBoundary)){

		adv<-datA[tmp[k,c_is]:tmp[k,c_if]]
		adv<-cutADV(adv,prRight=0.9,prLeft=0.05,centerBoundary=centerBoundary)
		tt<-which(adv>(tmp[k,c_cn]+1+tmp[k,c_adL])/2)
		if(length(tt)>50){sdH<-sd(adv[tt],na.rm=T)}
	}
}#k

if(length(sdH)>0){sdH<-mean(sdH,na.rm=T)}else{sdH<-NA}

out<-sdH

}

#########################################################################
##################
################## Function AllelicDiff for recognition
##################
#########################################################################


getADforGAPrecogn<-function(tmp,datA,sdH){

k<-37
for(k in 1:dim(tmp)[[1]]){

adv<-datA[tmp[k,c_is]:tmp[k,c_if]]
adv<-cutADV(adv,prRight=0.85,prLeft=0.05,centerBoundary=0.2)

if(length(adv)>15){
	
	hh<-hist(adv, breaks=seq(0,max(1,round(max(adv)+1)),0.1),right=F,plot=F)

	if(length(adv)>50){
	
		dd<-density(adv,n=20)
		
		MaxXY<-numeric(0)
		Id<-0

		for(s in 2:length(dd$x)){
			if((dd$y[s]<dd$y[s-1])&&(Id==0)){
				MaxXY<-c(MaxXY,s-1);Id<-1
				}else{if((dd$y[s]>dd$y[s-1])&&(Id==1)){Id<-0}}
		}#s

		if(sum(hh$counts[1:2])/sum(hh$counts)>0.2){
			if((hh$counts[1]>hh$counts[2]*1.2)&&(hh$counts[1]>hh$counts[3]*1.3)){
				tmp[k,c_adL]<-0
			}else{
				if((hh$counts[2]>hh$counts[1]*1.2)&&(hh$counts[2]>hh$counts[3]*1.2)){
					if(dd$x[MaxXY[1]]<0.3){tmp[k,c_adL]<-dd$x[MaxXY[1]]
					}else{tmp[k,c_adL]<-hh$mids[2]}
				}else{
					ll<-chooseMix2Model_new(x=adv, lambda=0.4, mu=c(quantile(adv,prob=0.2),quantile(adv,prob=0.8)),
						 sigsqrd=c(sdH^2,sdH^2), eps = 1e-02, maxit = 200)
					if(ll[3]>=0.2){tmp[k,c_adL]<-ll[1]
					}else{tmp[k,c_adL]<-dd$x[which.max(dd$y)]}	
					}
			}
		}else{
			
			if((length(MaxXY)==2)&&(MaxXY[2]>MaxXY[1]+4)
				&&(min(dd$y[(MaxXY[1]+2):(MaxXY[2]-2)])<0.8*min(dd$y[MaxXY[1]],dd$y[MaxXY[2]]))){

					if(dd$x[MaxXY[1]]<0.3){	tmp[k,c_adL]<-dd$x[MaxXY[2]]
					}else{tmp[k,c_adL]<-dd$x[MaxXY[1]]}
					
			}else{
				ll<-chooseMix2Model_new(x=adv, lambda=0.4, mu=c(quantile(adv,prob=0.2),quantile(adv,prob=0.8)),
					 sigsqrd=c(sdH^2,sdH^2), eps = 1e-02, maxit = 200)
				if(ll[3]>=0.2){tmp[k,c_adL]<-ll[1]
				}else{tmp[k,c_adL]<-dd$x[which.max(dd$y)]}	
				}
			}
	}else{tmp[k,c_adL]<-hh$mids[which.max(hh$counts)]}

}else{

if(length(adv)>0){
	hh<-hist(adv, breaks=seq(0,max(1,round(max(adv)+1)),0.2),plot=F)
	tmp[k,c_adL]<-hh$mids[which.max(hh$counts)]
}else{tmp[k,c_adL]<-NA}

}

}#k


tmp<-round(tmp,4)
out<-tmp

}#function



#########################################################################
##################
################## Function checkGermHomoSegments 
##################
#########################################################################

checkGermHomoSegments<-function(tmp,datA,Centr_CL,sdH){

for(k in 1:dim(tmp)[[1]]){

if((tmp[k,c_BA1]==0)&&(tmp[k,c_CN1]!=0)&&(tmp[k,c_len]>50)){

adv<-datA[tmp[k,c_is]:tmp[k,c_if]]
tt<-which(is.na(adv));if(length(tt)>0){adv<-adv[-tt]}
tt<-which(adv<0);if(length(tt)>0){adv<-adv[-tt]}
if(length(adv)>0){tt<-which(adv>Centr_CL[4,4]);if(length(tt)>0){adv<-adv[-tt]}}

}
}#k

out<-tmp
}#function


#########################################################################
##################
################## Function AllelicDiff confidence to distinguish balanced and not balanced segments
##################
#########################################################################


getADconfForBalanced<-function(tmp,datA,Centr_CL){

tmp[,c_nA]<-0

k<-9
for(k in 1:dim(tmp)[[1]]){

adv<-datA[tmp[k,c_is]:tmp[k,c_if]]
tt<-which(is.na(adv));if(length(tt)>0){adv<-adv[-tt]}
tt<-which(adv<0);if(length(tt)>0){adv<-adv[-tt]}
if(length(adv)>0){tt<-which(adv>Centr_CL[4,4]);if(length(tt)>0){adv<-adv[-tt]}}

if((length(adv)>50)&&(length(adv)>tmp[k,c_len]/10)){

	D<-0	
	if((tmp[k,c_nC]>=1)&&(tmp[k,c_nC]<=2)){D<-Centr_CL[4,2]/5}
		
	if((tmp[k,c_nC]>=2)&&(tmp[k,c_nC]<=4)){D<-Centr_CL[4,5]/5}

	if((tmp[k,c_nC]>=4)&&(tmp[k,c_nC]<=6)){D<-Centr_CL[4,10]/5}

	if((tmp[k,c_nC]>=6)&&(tmp[k,c_nC]<=8)){D<-Centr_CL[4,17]/5}

	if(D>0){
		hh<-hist(adv, breaks=seq(0,max(10*D,round(max(adv)+1)),D),right=F,plot=F)
		tmp[k,c_nA]<-max(hh$counts[4:7])/(sum(hh$counts[1:2])/2+1)

	for(p in 1:3){
	
	if((tmp[k,c_CN1+3*(p-1)]==2)&&(tmp[k,c_BA1+3*(p-1)]==1)&&(tmp[k,c_nA]>1))
		{tmp[k,c_rnk1+3*(p-1)]<-(-tmp[k,c_rnk1+3*(p-1)])}
		
	
	if((tmp[k,c_CN1+3*(p-1)]==4)&&(tmp[k,c_BA1+3*(p-1)]==2)&&(tmp[k,c_nA]>1))
		{tmp[k,c_rnk1+3*(p-1)]<-(-tmp[k,c_rnk1+3*(p-1)])}

	
	if((tmp[k,c_CN1+3*(p-1)]==6)&&(tmp[k,c_BA1+3*(p-1)]==3)&&(tmp[k,c_nA]>1))
		{tmp[k,c_rnk1+3*(p-1)]<-(-tmp[k,c_rnk1+3*(p-1)])}
	}#p


	}

}#length
}#k


tmp<-round(tmp,4)
out<-tmp

}#function


#########################################################################
##################
################## Function normalmixEM2compLaSDfit
##################
#########################################################################


normalmixEM2compLaSDfit<-function (x, lambda, mu, sigsqrd, eps = 1e-01, maxit = 20) 
{
    mi1<-c(mu[1],sigsqrd[1],lambda);names(mi1)<-c("mu","D","la")
    mi2<-c(mu[2],sigsqrd[2],1-lambda);names(mi2)<-c("mu","D","la")

    mx <- mean(x)
    const <- length(x) * 0.918938533204673
    dl <- 1 + eps
    iter <- 0
    a1 <- (x - mi1[1])^2
    b1 <- (2-sign(abs(mi1[1])))*(mi1[3]/sqrt(mi1[2])) * exp(-a1/2/mi1[2])
    a2 <- (x - mi2[1])^2
    b2 <- (mi2[3]/sqrt(mi2[2])) * exp(-a2/2/mi2[2])
    l<-sum(log(b1 + b2))
    ll <- c(mi1,mi2,l)
    postprobs <- b1/(b1 + b2)
    mi1[3] <- mean(postprobs)
    mi2[3]<- (1 - mi1[3])
  
    while (dl > eps && iter < maxit && mi1[3]*mi2[3]>0) {
        iter <- iter + 1
        mi1[2] <- mean(postprobs * a1)/mi1[3]
        mi2[2] <- mean((1 - postprobs) * a2)/mi2[3]

        a1 <- (x - mi1[1])^2
        b1 <- (2-sign(abs(mi1[1])))*(mi1[3]/sqrt(mi1[2])) * exp(-a1/2/mi1[2])
        a2 <- (x - mi2[1])^2
        b2 <- (mi2[3]/sqrt(mi2[2])) * exp(-a2/2/mi2[2])

        oldl <- l
        l <- sum(log(b1 + b2))
        dl <- l - oldl
        ll <- cbind(ll,c(mi1,mi2,l))

        postprobs <- b1/(b1 + b2)
        mi1[3] <- mean(postprobs)
	  mi2[3]<- (1 - mi1[3])
    }
  if (mi1[3]==0){ mi2[2] <- mean((x - mi2[1])^2)
			l<-logLike(x, Lambda=0, MU=c(mi1[1],mi2[1]), SD=c(sqrt(mi1[2]),sqrt(mi2[2])))
  	      	ll <- cbind(ll,c(mi1,mi2,l+const))
	
  }else{if (mi2[3]==0){ mi1[2] <- mean((x - mi1[1])^2)
			l<-logLike(x, Lambda=1, MU=c(mi1[1],mi2[1]), SD=c(sqrt(mi1[2]),sqrt(mi2[2])))
  	      	ll <- cbind(ll,c(mi1,mi2,l+const))
	
		  }
		}




  ll[2,]<-sqrt(ll[2,])
  ll[5,]<-sqrt(ll[5,])
  ll[7,]<-ll[7,]-const 
  out <- round(ll[,which.max(ll[7,])],3)
  out
}

#tmp<-TMP


#########################################################################
##################
################## Function normalmixEM2compLafit
##################
#########################################################################


normalmixEM2compLafit<-function (x, lambda, mu, sigsqrd, eps = 1e-01, maxit = 20) 
{
    mi1<-c(mu[1],sigsqrd[1],lambda);names(mi1)<-c("mu","D","la")
    mi2<-c(mu[2],sigsqrd[2],1-lambda);names(mi2)<-c("mu","D","la")

    mx <- mean(x)
    const <- length(x) * 0.918938533204673
    dl <- 1 + eps
    iter <- 0
    a1 <- (x - mi1[1])^2
    a11 <- (-x - mi1[1])^2
    b1 <- (mi1[3]/sqrt(mi1[2])) * (exp(-a1/2/mi1[2])+exp(-a11/2/mi1[2]))
    a2 <- (x - mi2[1])^2
    b2 <- (mi2[3]/sqrt(mi2[2])) * exp(-a2/2/mi2[2])
    l<-sum(log(b1 + b2))
    ll <- c(mi1,mi2,l)
    postprobs <- b1/(b1 + b2)
    mi1[3] <- mean(postprobs)
    mi2[3]<- (1 - mi1[3])

   
    while (dl > eps && iter < maxit && mi1[3]*mi2[3]>0) {
        iter <- iter + 1
  
        a1 <- (x - mi1[1])^2
        a11 <- (-x - mi1[1])^2
        b1 <- (mi1[3]/sqrt(mi1[2])) * (exp(-a1/2/mi1[2])+exp(-a11/2/mi1[2]))
        a2 <- (x - mi2[1])^2
        b2 <- (mi2[3]/sqrt(mi2[2])) * exp(-a2/2/mi2[2])

        oldl <- l
        l <- sum(log(b1 + b2))
        dl <- l - oldl
        ll <- cbind(ll,c(mi1,mi2,l))

        postprobs <- b1/(b1 + b2)
        mi1[3] <- mean(postprobs)
	  mi2[3]<- (1 - mi1[3])
    }
  if (mi1[3]==0){ l<-logLike(x, Lambda=0, MU=c(mi1[1],mi2[1]), SD=c(sqrt(mi1[2]),sqrt(mi2[2])))
  	      	ll <- cbind(ll,c(mi1,mi2,l+const))
	
  }else{if (mi2[3]==0){l<-logLike(x, Lambda=1, MU=c(mi1[1],mi2[1]), SD=c(sqrt(mi1[2]),sqrt(mi2[2])))
  	      	ll <- cbind(ll,c(mi1,mi2,l+const))
	
		  }
		}

  ll[2,]<-sqrt(ll[2,])
  ll[5,]<-sqrt(ll[5,])
  ll[7,]<-ll[7,]-const 
  out <- round(ll[,which.max(ll[7,])],3)
  out
}



#########################################################################
##################
################## Function CNandGTbyLogLike_withSD
##################
#########################################################################


CNandGTbyLogLike_withSD_Rank<-function(tmp,datA,Centr_CL,sdH,Tolerance){

c_CL<-cbind(Centr_CL,Centr_CL[,which(Centr_CL[1,]-Centr_CL[2,]==0)])
c_CL[2,26:34]<-0
c_CL[3,26:34]<-c_CL[1,26:34]+30
c_CL[4,26:34]<-c_CL[5,26:34]+1
c_CL<-rbind(c_CL,c_CL[1:4,])
rownames(c_CL)[6:9]<-c("sd_L","sd_H","LogLike","Lambda")

c_CL[6:9,]<-0
tmp[,c_CN1:(c_CN1+1)]<-(-1)
tmp[,c_conf]<-0
tmp[,c_adH]<-0

##################  Estimating variance in AD bands

for(k in 1:dim(tmp)[[1]]){

	adv<-datA[tmp[k,c_is]:tmp[k,c_if]]
	adv<-cutADV(adv,prRight=0.95,prLeft=0.01,centerBoundary=0.2)

	if(length(adv)>150){

		jj<-which(abs(c_CL[1,]-tmp[k,c_nC])<1)

		for(j in 1:length(jj)){
			if(c_CL[2,jj[j]]!=0){Lambda<-0.4}else{Lambda<-0}
			ll<-normalmixEM2compLaSDfit(x=adv, lambda=Lambda, mu=c(c_CL[4,jj[j]],c_CL[5,jj[j]]+1), sigsqrd=c((sdH/1.5)^2,(sdH/1.5)^2), eps = 0.1, maxit = 20) 
			c_CL[6,jj[j]]<-ll[2]
			c_CL[7,jj[j]]<-ll[5]
			c_CL[8,jj[j]]<-ll[7]
			c_CL[9,jj[j]]<-ll[3]
		}

		tt<-order(c_CL[8,jj],decreasing=T)
		tt<-jj[tt]

		if(length(tt)>1){
			j<-2
			while((j<=length(tt))&&(routhEqual(c_CL[8,tt[1]],c_CL[8,tt[j]],0,Tolerance=0.2))){j<-j+1}
			if((j<=2)&&(c_CL[9,tt[1]]>0.2)){
				tmp[k,c_CN1]<-c_CL[1,tt[1]]
				tmp[k,c_BA1]<-c_CL[2,tt[1]]
				tmp[k,c_conf]<-c_CL[6,tt[1]]
				tmp[k,c_adH]<-c_CL[7,tt[1]]
			}
		}

	}# if length(adv)
}#k

tt<-which(tmp[,c_CN1]>0)

c_CL[6:9,]<-0

for(j in 1:dim(c_CL)[[2]]){
	jj<-which((abs(tmp[tt,c_CN1]-c_CL[1,j])+abs(tmp[tt,c_BA1]-c_CL[2,j]))==0)
	if(length(jj)>10){
		c_CL[6,j]<-median(tmp[tt[jj],c_conf])
		c_CL[7,j]<-median(tmp[tt[jj],c_adH])
	}
}


tt<-which(c_CL[6,]>0)
if(length(tt)>0){
	jj<-which(c_CL[6,]==0)
	if(length(jj)>0){c_CL[6,jj]<-mean(c_CL[6,tt]);c_CL[7,jj]<-mean(c_CL[7,tt])}
}else{c_CL[6,]<-sdH/1.5;c_CL[7,]<-sdH/1.5}

tmp[,c_conf]<-0
tmp[,c_adH]<-0

################ Choosing CN and GT by logLikelyhood

tmp[,c_CN1:(c_CN1+8)]<-0
tmp[,c_CN1+3]<-(-1);tmp[,c_CN1+6]<-(-1)
tmp[,c_rnk1]<-1;tmp[,c_rnk1+3]<-2;tmp[,c_rnk1+6]<-3


k<-277
for(k in 1:dim(tmp)[[1]]){

	adv<-datA[tmp[k,c_is]:tmp[k,c_if]]
	adv<-cutADV(adv,prRight=0.95,prLeft=0.05,centerBoundary=0.2)

	if(length(adv)>10){

		jj<-which(abs(c_CL[1,]-tmp[k,c_nC])<1)

		for(j in 1:length(jj)){
			if(c_CL[2,jj[j]]!=0){Lambda<-0.4}else{Lambda<-0}
			ll<-normalmixEM2compLafit(x=adv, lambda=Lambda, mu=c(c_CL[4,jj[j]],c_CL[5,jj[j]]+1), sigsqrd=c(c_CL[6,jj[j]]^2,c_CL[7,jj[j]]^2), eps = 0.1, maxit = 20) 
			c_CL[8,jj[j]]<-ll[7]
			c_CL[9,jj[j]]<-ll[3]
		}

		tt<-order(c_CL[8,jj],decreasing=T)
		tt<-jj[tt]

		for(p in 1:min(length(tt),3)){
			tmp[k,c_CN1+3*(p-1)]<-c_CL[1,tt[p]]
			tmp[k,c_BA1+3*(p-1)]<-c_CL[2,tt[p]]
			if(c_CL[9,tt[p]]<0.1){tmp[k,c_BA1+3*(p-1)]<-0}
		}

		if(length(tt)>1){
			j<-2
			while((j<=length(tt))&&(routhEqual(c_CL[8,tt[1]],c_CL[8,tt[j]],0,Tolerance))){j<-j+1}
			if(j>2){tmp[k,c_rnk1+3]<-1}
			if(j>3){tmp[k,c_rnk1+6]<-1}
		}

}# if length(adv)

}#k

out<-tmp
}# function CNandGTbyLogLike_withSD_Rank



########################################################################
############
############  Function assignCNandGT: choose interpretation based on CN, ranked combintaion of
############  CN and GT, and discriminating balanced and not balanced regions
############
########################################################################



assignCNandGTstable<-function(tmp,Length){

tmp[,c_CN]<-0
tmp[,c_BA]<-0
k<-17
for(k in 1:dim(tmp)[[1]]){
	p<-0
	if(tmp[k,c_len]<Length){
	p<-1

	}else{
	if(abs(tmp[k,c_nC]-tmp[k,c_CN1])<0.7){
		if(tmp[k,c_rnk1]<0){
			if((tmp[k,c_rnk1+3]==1)&&(abs(tmp[k,c_nC]-tmp[k,c_CN1+3])<0.7)){p<-2
			}else{if((tmp[k,c_rnk1+6]==1)&&(abs(tmp[k,c_nC]-tmp[k,c_CN1+6])<0.7)){p<-3
				}else{p<-1}
				}
		
		}else{p<-1}
	}else{if((tmp[k,c_rnk1]>0)&&(tmp[k,c_rnk1+3]>1)){p<-1
		}else{if(tmp[k,c_rnk1]<0){
				if((tmp[k,c_rnk1+3]==1)&&(abs(tmp[k,c_nC]-tmp[k,c_CN1+3])<0.7)){p<-2
				}else{if((tmp[k,c_rnk1+6]==1)&&(abs(tmp[k,c_nC]-tmp[k,c_CN1+6])<0.7)){p<-3
					}else{p<-1}
					}
			}else{if((tmp[k,c_rnk1+3]==1)&&(abs(tmp[k,c_nC]-tmp[k,c_CN1+3])<0.7)){p<-2
				}else{if((tmp[k,c_rnk1+6]==1)&&(abs(tmp[k,c_nC]-tmp[k,c_CN1+6])<0.7)){p<-3}
					}
				}
			}
		}
	}#else >Length
	if(p==0){p<-1}
	tmp[k,c_CN]<-tmp[k,c_CN1+3*(p-1)];tmp[k,c_BA]<-tmp[k,c_BA1+3*(p-1)]
}#k


tt<-which(tmp[,c_len]>=Length)
tt1<-table(tmp[,c_chr])

if(length(which(tt1==1))>0){tt1<-tt1[which(tt1==1)]
	tt1<-as.numeric(rownames(tt1))
	for(j in 1:length(tt1)){
		tt<-c(tt,which(tmp[,c_chr]==tt1[j]))
	}
tt<-as.numeric(rownames(table(tt)))
}


tmp[,c_conf]<-0

if(length(tt)>1){tmpS<-tmp[-tt,];tmpB<-tmp[tt,]

tmpB<-UniteSEG_TMP(tmpB)

for(k in 1:dim(tmpB)[[1]]){
	tt<-which(tmp[,c_is]>=tmpB[k,c_is])
	if(length(tt)>0){kk<-which(tmp[tt,c_if]<=tmpB[k,c_if])
	if(length(kk)>0){
		tmp[tt[kk],c_CN]<-tmpB[k,c_CN]
		tmp[tt[kk],c_BA]<-tmpB[k,c_BA]
		tmp[tt[kk],c_conf]<-k
	}}
}#k

}else{tmpS<-tmp}

k<-1
for(k in 1:dim(tmpS)[[1]]){
	tt<-which(tmp[,c_ind]==tmpS[k,c_ind])
	if(length(tt)==1){
		tmp[tt,c_CN]<-tmpS[k,c_CN]
		tmp[tt,c_BA]<-tmpS[k,c_BA]
		tmp[tt,c_conf]<-(-k)}
}




out<-tmp

}# function






########################################################################
############
############  Function Manage CN profile: extracting true breaks
############
########################################################################

trueCNbreakCount_new<-function(tmp,Break){

tmp[,c_ind]<-seq(1,dim(tmp)[[1]])

tmpB<-tmp

tt<-which(tmpB[,c_len]>=50)
tt1<-table(tmpB[,c_chr])

if(length(which(tt1==1))>0){tt1<-tt1[which(tt1==1)]
	tt1<-as.numeric(names(tt1))
	for(j in 1:length(tt1)){
		tt<-c(tt,which(tmpB[,c_chr]==tt1[j]))
	}
tt<-as.numeric(names(table(tt)))
}

if(length(tt)>0){
if(dim(tmpB)[[1]]-length(tt)>1){tmpS<-tmpB[-tt,]}else{tmpS<-rbind(tmpB[-tt,],tmpB[-tt,])}

tmpB<-tmpB[tt,]


tmpB[,c_cnBP]<-0
tmpB[,c_CN]<-tmpB[,c_nC]

for(k in 2:dim(tmpB)[[1]]){if(abs(tmpB[k,c_cn]-tmpB[k-1,c_cn])>Break){tmpB[k-1,c_cnBP]<-1}}

for(j in 2:10){
for(k in (j+1):dim(tmpB)[[1]]){
	if((abs(tmpB[k,c_cn]-tmpB[k-j,c_cn])>Break) && (max(tmpB[(k-j):(k-1),c_cnBP])==0)){tmpB[k-j,c_cnBP]<-j}
}
}

kk<-0
for(k in 1:dim(tmpB)[[1]]){
	if(tmpB[k,c_cnBP]>0){kk<-kk+1;for(j in 1:tmpB[k,c_cnBP]){tmpB[k+j-1,c_cnBP]<-(-kk)}}
}

kk1<-1
for(k in 1:dim(tmpB)[[1]]){
	if(tmpB[k,c_cnBP]!=0){kk2<-k;if(kk2>kk1){tmpB[kk1:kk2,c_CN]<-round(weighted.mean(tmpB[kk1:kk2,c_nC],tmpB[kk1:kk2,c_len]/sum(tmpB[kk1:kk2,c_len])),1)}
		kk1<-kk2+1}
}

for(k in 2:(dim(tmpB)[[1]]-1)){
	if((tmpB[k,c_cnBP]!=0)&&(tmpB[k+1,c_cnBP]!=tmpB[k,c_cnBP])&&(tmpB[k-1,c_cnBP]!=tmpB[k,c_cnBP])&&(abs(tmpB[k,c_CN]-tmpB[k+1,c_CN])>=1)){tmpB[k,c_cnBP]<-abs(tmpB[k,c_cnBP])}
}


kk1<-1
for(k in 1:dim(tmpB)[[1]]){
	if(tmpB[k,c_cnBP]>=0){kk2<-k
		if((kk2>kk1+1)&&(tmpB[kk2,c_CN]==tmpB[kk1,c_CN])&&(max(abs(tmpB[kk1:(kk2-1),c_CN]-tmpB[kk2,c_CN]))<0.7)){
			tmpB[kk1:(kk2-1),c_cnBP]<-0
			tmpB[kk1:(kk2-1),c_CN]<-tmpB[kk2,c_CN]
		}else{
			if((kk2==kk1+1)&&(tmpB[kk2,c_CN]==tmpB[kk1,c_CN])&&(abs(tmpB[kk1,c_CN]-tmpB[kk2,c_CN])<0.7)){
			tmpB[kk1,c_cnBP]<-0
			tmpB[kk1,c_CN]<-tmpB[kk2,c_CN]}
			}
	kk1<-kk2+1
	}
}

tt<-which(tmpB[,c_CN]>5)
if(length(tt)>0){tmpB[tt,c_CN]<-round(tmpB[tt,c_CN])

tmp[tmpB[tt,c_ind],c_nC]<-tmpB[tt,c_CN]


tmpB[,c_CN]<-round(tmpB[,c_CN])
tmpB<-UniteSEG_TMP(tmpB)
tmpB[,c_conf]<-seq(1,dim(tmpB)[[1]])
tmpS[,c_conf]<-(-1*seq(1,dim(tmpS)[[1]]))

for(k in 1:dim(tmpB)[[1]]){
	tt<-which(tmp[,c_is]>=tmpB[k,c_is])
	if(length(tt)>0){kk<-which(tmp[tt,c_if]<=tmpB[k,c_if])
	if(length(kk)>0){tmp[tt[kk],c_conf]<-tmpB[k,c_conf]
	}}
}#k

tmp[tmpS[,c_ind],c_conf]<-tmpS[,c_conf]




}# if CN>5 present in tmpB

}# if 

tmp<-round(tmp,4)
tmp[,c_nC]<-round(tmp[,c_nC],1)

out<-tmp

}#function




########################################################################
############
############  Function Manage CN profile: extracting true breaks
############
########################################################################

fitCNbreakCount_short<-function(tmp){

recCHR<-CHR_table_recognition()

checkSEGfitCN<-function(ss,bb){
	if((abs(tmpS[ss,c_nC]-tmpB[bb,c_CN])<0.9)&&(tmpB[bb,c_CN]>=5)&&(tmpS[ss,c_nC]>=4.5))
	{FIT<-T}else{FIT<-F}
	out<-FIT
}

tmp[,c_ind]<-seq(1,dim(tmp)[[1]])

############ Fitting small segments with high CN


tt<-which(tmp[,c_conf]<0)

if(length(tt)>0){

tt<-matrix(0,2,2)
t_pos<-0
t_chr<-1
k_neg<-0


for(k in 1:dim(tmp)[[1]]){
  if(tmp[k,c_chr]==t_chr){
	if(tmp[k,c_conf]<0){k_neg<-k_neg+1
		  if(k==dim(tmp)[[1]]){if(!is.na(t_pos)){tt<-cbind(tt,c(t_pos,k_neg))}}

	}else{if(k_neg>0){
		if(is.na(t_pos)){tt<-cbind(tt,c(tmp[k,c_conf],k_neg));t_pos<-tmp[k,c_conf]
		}else{if(tmp[k,c_conf]==t_pos){tt<-cbind(tt,c(t_pos,k_neg))
			}else{tt<-cbind(tt,c((t_pos+tmp[k,c_conf])/2,k_neg));t_pos<-tmp[k,c_conf]}
			}
		k_neg<-0
		}else{t_pos<-tmp[k,c_conf]}
		}
  }else{if(k_neg>0){
		if(!is.na(t_pos)){tt<-cbind(tt,c(t_pos,k_neg))
		}else{if(k<dim(tmp)[[1]]){kk<-which(tmp[k:dim(tmp)[[1]],c_conf]>0)}else{kk<-numeric(0)}
			if(length(kk)>0){tmp[k+kk-1,c_conf]<-tmp[k+kk-1,c_conf]+k_neg
				tmp[k-1,c_conf]<-tmp[k+kk[1]-1,c_conf]-1
				for(p in 2:k_neg){tmp[k-p,c_conf]<-tmp[k-p+1,c_conf]-1}
			}else{tmp[k-1,c_conf]<-max(tmp[,c_conf])+k_neg
				if(k_neg>1){
				for(p in 2:k_neg){tmp[k-p,c_conf]<-tmp[k-p+1,c_conf]-1}
				}
				}
			}		
		k_neg<-0
	  }
	  t_pos<-NA;t_chr<-tmp[k,c_chr]
	  if(tmp[k,c_conf]<0){k_neg<-k_neg+1}else{t_pos<-tmp[k,c_conf]}
  	  if(k==dim(tmp)[[1]]){
		if(k_neg>0){
			if(!is.na(t_pos)){tt<-cbind(tt,c(t_pos,k_neg))
			}else{tmp[k,c_conf]<-max(tmp[,c_conf])+k_neg}		
		}
	   }

	}
}#k

if(dim(tt)[[2]]>2){

if(dim(tt)[[2]]>4){tt<-tt[,-c(1,2)];ONE<-F}else{tt<-cbind(tt[,-c(1,2)],tt[,-c(1,2)]);ONE<-T}
tt<-data.matrix(tt)
colnames(tt)<-seq(1,dim(tt)[[2]])

kk<-which(tmp[,c_conf]>0)
if(length(kk)>0){tmpS<-tmp[-kk,]
if((dim(tmp)[[1]]-length(kk)==1)||(ONE)){tmpS<-rbind(tmpS,tmpS)}
tmpB<-tmp[kk,]
}


tmpB[1,c_is]<-1

for(k in 2:dim(tmpB)[[1]]){
	if(tmpB[k-1,c_chr]!=tmpB[k,c_chr]){
		kk<-which(as.numeric(names(recCHR))==tmpB[k-1,c_chr])
		tmpB[k-1,c_if]<-recCHR[kk]
		tmpB[k,c_is]<-recCHR[kk]+1
	}
}
tmpB[,c_CN]<-round(tmpB[,c_nC])
tmpB<-shrinkReprTMP(tmpB)


tt1<-which(tt[1,]==0.5);if(length(tt1)>0){tt[1,tt1]<-1}

tmpS[,c_nS]<-1
tt_end<-length(which(tt[1,]<=dim(tmpB)[[1]]))
if(ONE&&(tt[1,]<=dim(tmpB)[[1]])){tt_end<-1}

TT<-1
j<-10
for(j in 1:tt_end){
	if(trunc(tt[1,j])==tt[1,j]){
		if(tt[2,j]==1){
			if(checkSEGfitCN(TT,tt[1,j])){tmp[tmpS[TT,c_ind],c_nC]<-tmpB[tt[1,j],c_CN]}
		}else{p<-2
				for(p in TT:(TT+tt[2,j]-1)){
				if(checkSEGfitCN(p,tt[1,j])){tmp[tmpS[p,c_ind],c_nC]<-tmpB[tt[1,j],c_CN]}
			}}
	}else{ kk<-trunc(tt[1,j])
		if(tt[2,j]==1){
			p<-which.min(c(abs(tmpS[TT,c_nC]-tmpB[kk,c_CN]),abs(tmpS[TT,c_nC]-tmpB[kk+1,c_CN])))
			if((p==1)&&(checkSEGfitCN(TT,kk))){tmp[tmpS[TT,c_ind],c_nC]<-tmpB[kk,c_CN]}
			if((p==2)&&(checkSEGfitCN(TT,kk+1))){tmp[tmpS[TT,c_ind],c_nC]<-tmpB[kk+1,c_CN]}

		}else{
			p<-which.min(c(abs(tmpS[TT,c_nC]-tmpB[kk,c_CN]),abs(tmpS[TT,c_nC]-tmpB[kk+1,c_CN])))
			if((p==1)&&(checkSEGfitCN(TT,kk))){tmp[tmpS[TT,c_ind],c_nC]<-tmpB[kk,c_CN]}
			p<-which.min(c(abs(tmpS[(TT+tt[2,j]-1),c_nC]-tmpB[kk,c_CN]),abs(tmpS[(TT+tt[2,j]-1),c_nC]-tmpB[kk+1,c_CN])))
			if((p==2)&&(checkSEGfitCN((TT+tt[2,j]-1),kk+1))){tmp[tmpS[(TT+tt[2,j]-1),c_ind],c_nC]<-tmpB[kk+1,c_CN]}
			}
		}
TT<-TT+tt[2,j]
}#j


}# if exists
}# if exists
out<-tmp

}#function



###########################################################
####
####   Function UniteSEGtmp_new Uniting segments with similar attribution
####
###########################################################

UniteSEG_TMP<-function(tmp){

FL<-T
while(FL==T){
FL<-F


for(k in 1:(dim(tmp)[[1]]-1)){

	if((tmp[k,c_chr]==tmp[k+1,c_chr])&&(trunc(tmp[k,c_CN])==tmp[k,c_CN])
		&&(tmp[k,c_CN]==tmp[k+1,c_CN])&&(tmp[k,c_BA]==tmp[k+1,c_BA])){

		tmp[k+1,c_is]<-tmp[k,c_is]
		tmp[k+1,c_isCN]<-tmp[k,c_isCN]

		w<-c(tmp[k+1,c_len],tmp[k,c_len])
		tmp[k+1,c_cn]<-weighted.mean(c(tmp[k+1,c_cn],tmp[k,c_cn]),w)
		tmp[k+1,c_nC]<-weighted.mean(c(tmp[k+1,c_nC],tmp[k,c_nC]),w)

		tmp[k+1,c_CN1:(c_CN1+8)]<-(-1)
		tmp[k+1,c_CN1]<-tmp[k+1,c_CN]
		tmp[k+1,c_BA1]<-tmp[k+1,c_BA]
		tmp[k+1,c_rnk1]<-1

		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k+1,c_conf]<-max(tmp[k+1,c_conf],tmp[k,c_conf])	
		tmp[k,c_ind]<-0
	}else{

	if((tmp[k,c_chr]==tmp[k+1,c_chr])&&(tmp[k,c_CN]==0)&&(tmp[k+1,c_CN]==0)){

		tmp[k+1,c_is]<-tmp[k,c_is]
		tmp[k+1,c_isCN]<-tmp[k,c_isCN]

		w<-c(tmp[k+1,c_len],tmp[k,c_len])
		tmp[k+1,c_cn]<-weighted.mean(c(tmp[k+1,c_cn],tmp[k,c_cn]),w)
		
		tmp[k+1,c_nC]<-weighted.mean(c(tmp[k+1,c_nC],tmp[k,c_nC]),w)

		tmp[k+1,c_CN1:(c_CN1+8)]<-(-1)
		tmp[k+1,c_CN1]<-tmp[k+1,c_CN]
		tmp[k+1,c_BA1]<-tmp[k+1,c_BA]
		tmp[k+1,c_rnk1]<-1

		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k+1,c_conf]<-max(tmp[k+1,c_conf],tmp[k,c_conf])	
		tmp[k,c_ind]<-0
	}
}
}#k

tt<-which(tmp[,c_ind]==0);if(length(tt)!=0){tmp<-tmp[-tt,];FL<-T}

} #FL

tmp<-round(tmp,4)
tmp[,c_nC]<-round(tmp[,c_nC],1)

out<-tmp

}# function








###########################################################
####
####   Function shrinkReprTMP Uniting segments with equal c_conf column
####
###########################################################

shrinkReprTMP<-function(tmp){

tmp[1,c_cn]<-tmp[1,c_len]*tmp[1,c_cn]
for(k in 1:(dim(tmp)[[1]]-1)){
	if(tmp[k,c_conf]==tmp[k+1,c_conf]){
		tmp[k+1,c_is]<-tmp[k,c_is]
		tmp[k+1,c_isCN]<-tmp[k,c_isCN]

		tmp[k+1,c_cn]<-tmp[k,c_cn]+tmp[k+1,c_len]*tmp[k+1,c_cn]
		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k+1,c_CN1:(c_CN1+8)]<-(-1)
		tmp[k+1,c_CN1]<-tmp[k+1,c_CN]
		tmp[k+1,c_BA1]<-tmp[k+1,c_BA]
		tmp[k+1,c_rnk1]<-1

		tmp[k,1]<-0
		FL<-T
	}else{tmp[k,c_cn]<-tmp[k,c_cn]/tmp[k,c_len]
		tmp[k+1,c_cn]<-tmp[k+1,c_len]*tmp[k+1,c_cn]
		}

}#k
k<-dim(tmp)[[1]]
tmp[k,c_cn]<-tmp[k,c_cn]/tmp[k,c_len]

tt<-which(tmp[,1]==0);if(length(tt)!=0){tmp<-tmp[-tt,]}


tmp<-round(tmp,4)
tmp[,c_nC]<-round(tmp[,c_nC],1)

out<-tmp

}# function



###########################################################
####
####   Function adjustTMPstableSubclone regions with intermediate CN fitted to exact neihbours
####
###########################################################


adjustTMPstableSubclone<-function(tmp,recCHR){

#tmp<-TMP

checkSEGfit<-function(ss,bb){

FIT<-F
if(abs(tmpS[ss,c_nC]-tmpB[bb,c_nC])<0.7){
	if((tmpS[ss,c_CN1]==tmpB[bb,c_CN])&&(tmpS[ss,c_BA1]==tmpB[bb,c_BA])&&(tmpS[ss,c_rnk1]==1))
		{FIT<-T}
	if((tmpS[ss,c_CN1+3]==tmpB[bb,c_CN])&&(tmpS[ss,c_BA1+3]==tmpB[bb,c_BA])&&(tmpS[ss,c_rnk1+3]==1))
		{FIT<-T}
	if((tmpS[ss,c_CN1+6]==tmpB[bb,c_CN])&&(tmpS[ss,c_BA1+6]==tmpB[bb,c_BA])&&(tmpS[ss,c_rnk1+6]==1))
		{FIT<-T}
}
out<-FIT
}#k

############# main

FL<-T
while(FL==T){
FL<-F

ttt<-which(tmp[,c_conf]<0)

if(length(ttt)>0){


tt<-matrix(0,2,2)
t_pos<-0
t_chr<-1
k_neg<-0


for(k in 1:dim(tmp)[[1]]){
  if(tmp[k,c_chr]==t_chr){
	if(tmp[k,c_conf]<0){k_neg<-k_neg+1
		  if(k==dim(tmp)[[1]]){if(!is.na(t_pos)){tt<-cbind(tt,c(t_pos,k_neg))}}

	}else{if(k_neg>0){
		if(is.na(t_pos)){tt<-cbind(tt,c(tmp[k,c_conf],k_neg));t_pos<-tmp[k,c_conf]
		}else{if(tmp[k,c_conf]==t_pos){tt<-cbind(tt,c(t_pos,k_neg))
			}else{tt<-cbind(tt,c((t_pos+tmp[k,c_conf])/2,k_neg));t_pos<-tmp[k,c_conf]}
			}
		k_neg<-0
		}else{t_pos<-tmp[k,c_conf]}
		}
  }else{if(k_neg>0){
		if(!is.na(t_pos)){tt<-cbind(tt,c(t_pos,k_neg))
		}else{if(k<dim(tmp)[[1]]){kk<-which(tmp[k:dim(tmp)[[1]],c_conf]>0)}else{kk<-numeric(0)}
			if(length(kk)>0){tmp[k+kk-1,c_conf]<-tmp[k+kk-1,c_conf]+k_neg
				tmp[k-1,c_conf]<-tmp[k+kk[1]-1,c_conf]-1
				for(p in 2:k_neg){tmp[k-p,c_conf]<-tmp[k-p+1,c_conf]-1}
			}else{tmp[k-1,c_conf]<-max(tmp[,c_conf])+k_neg
				if(k_neg>1){
				for(p in 2:k_neg){tmp[k-p,c_conf]<-tmp[k-p+1,c_conf]-1}
				}
				}
			}		
		k_neg<-0
	  }
	  t_pos<-NA;t_chr<-tmp[k,c_chr]
	  if(tmp[k,c_conf]<0){k_neg<-k_neg+1}else{t_pos<-tmp[k,c_conf]}
  	  if(k==dim(tmp)[[1]]){
		if(k_neg>0){
			if(!is.na(t_pos)){tt<-cbind(tt,c(t_pos,k_neg))
			}else{tmp[k,c_conf]<-max(tmp[,c_conf])+k_neg}		
		}
	   }

	}
}#k


if(dim(tt)[[2]]>2){

if(dim(tt)[[2]]>4){tt<-tt[,-c(1,2)];ONE<-F}else{tt<-cbind(tt[,-c(1,2)],tt[,-c(1,2)]);ONE<-T}
tt<-data.matrix(tt)
colnames(tt)<-seq(1,dim(tt)[[2]])

kk<-which(tmp[,c_conf]>0)
if(length(kk)>0){tmpS<-tmp[-kk,]
if((dim(tmp)[[1]]-length(kk)==1)||(ONE)){tmpS<-rbind(tmpS,tmpS)}
tmpB<-tmp[kk,]
}

tmpB[1,c_is]<-1

for(k in 2:dim(tmpB)[[1]]){
	if(tmpB[k-1,c_chr]!=tmpB[k,c_chr]){
		kk<-which(as.numeric(names(recCHR))==tmpB[k-1,c_chr])
		tmpB[k-1,c_if]<-recCHR[kk]
		tmpB[k,c_is]<-recCHR[kk]+1
	}
}
k<-dim(tmpB)[[1]]
kk<-which(as.numeric(names(recCHR))==tmpB[k,c_chr])
tmpB[k,c_if]<-recCHR[kk]


tmpB<-shrinkReprTMP(tmpB)

tt1<-which(tt[1,]==0.5)
if(length(tt1)>0){tt[1,tt1]<-1}

tmpS[,c_nS]<-1
tt_end<-length(which(tt[1,]<=dim(tmpB)[[1]]))


TT<-1
j<-1
for(j in 1:tt_end){
	if(trunc(tt[1,j])==tt[1,j]){

		if(tt[2,j]==1){if(checkSEGfit(TT,tt[1,j])){tmpS[TT,c_conf]<-0}
		}else{ for(p in TT:(TT+tt[2,j]-1)){	if(checkSEGfit(p,tt[1,j])){tmpS[p,c_conf]<-0}}}
		
	}else{kk<-trunc(tt[1,j])
		if(kk==0){kk<-1
			if(tt[2,j]==1){if(checkSEGfit(TT,kk)){tmpS[TT,c_conf]<-kk}
			}else{ for(p in TT:(TT+tt[2,j]-1)){	if(checkSEGfit(p,kk)){tmpS[p,c_conf]<-kk}}}
		
		}else{if(tt[2,j]==1){
				if(checkSEGfit(TT,kk)){tmpS[TT,c_conf]<-kk}
				if(checkSEGfit(TT,kk+1)){
					if(tmpS[TT,c_conf]>0){
						if(abs(tmpS[TT,c_cn]-tmpB[kk,c_cn])>abs(tmpS[TT,c_cn]-tmpB[kk+1,c_cn]))
							{tmpS[TT,c_conf]<-kk+1}else{}
					}else{tmpS[TT,c_conf]<-kk+1}
				}else{}
			}else{p<-TT;if(checkSEGfit(p,kk)){tmpS[p,c_conf]<-kk}
				p<-(TT+tt[2,j]-1);if(checkSEGfit(p,kk+1)){tmpS[p,c_conf]<-kk+1}
				}
				
		}# not a first segment
	}# in between segments

	TT<-TT+tt[2,j]
}#j


if(length(which(tmpS[,c_conf]<0))>0){FL<-T}

ZERO<-F
tt<-which(tmpS[,c_conf]==0)
if(length(tt)>0){
	if(dim(tmpS)[[1]]-length(tt)>1){tmpS<-tmpS[-tt,]
	}else{if(dim(tmpS)[[1]]==length(tt)){ZERO<-T}else{tmpS<-tmpS[-tt,];tmpS<-rbind(tmpS,tmpS)}
		}
}else{FL<-F}

if((!ZERO)&&(length(which(tmpS[,c_conf]>0))>0)){FL<-T}

tmp[,c_conf]<-0

for(k in 1:dim(tmpB)[[1]]){
	tt<-which(tmp[,c_is]>=tmpB[k,c_is])
	if(length(tt)>0){kk<-which(tmp[tt,c_if]<=tmpB[k,c_if])
	if(length(kk)>0){
		tmp[tt[kk],c_CN]<-tmpB[k,c_CN]
		tmp[tt[kk],c_BA]<-tmpB[k,c_BA]
		tmp[tt[kk],c_conf]<-tmpB[k,c_conf]
	}}
}#k

if(!ZERO){
for(k in 1:dim(tmpS)[[1]]){
	kk<-which(tmp[,c_ind]==tmpS[k,c_ind])
	if(length(kk)==1){
		if(tmpS[k,c_conf]>0){
			tmp[kk,c_CN]<-tmpB[tmpS[k,c_conf],c_CN]
			tmp[kk,c_BA]<-tmpB[tmpS[k,c_conf],c_BA]
			tmp[kk,c_conf]<-tmpB[tmpS[k,c_conf],c_conf]
		}else{tmp[kk,c_CN]<-tmpS[k,c_CN]
			tmp[kk,c_BA]<-tmpS[k,c_BA]
			tmp[kk,c_conf]<-tmpS[k,c_conf]
			}
	}
}
}

tmp<-shrinkReprTMP(tmp)

#if proper subclones
}else{FL<-F}

# if subclones

}else{FL<-F}

}#FL


tmp<-round(tmp,4)
tmp[,c_nC]<-round(tmp[,c_nC],1)

out<-tmp

}#function



###########################################################
####
####   Function iterativeSegmentsFit: segments are iterativelly fitted, 
####     if possible, to stable neihbouring segments
####   Iterations are performed according to the length of the segments
####   We used 1000 SNPs as a basis segment length, going further to 800,400,200,100 SNPs
####   Other length combinations were not tested. 
####
###########################################################


iterativeSegmentsFit<-function(tmp,segLength){

recCHR<-CHR_table_recognition()

tmp<-assignCNandGTstable(tmp,Length=segLength[1])

#### Cycle to fit segments of >800,>400,>200,>100 length in SNP counts

p<-1
for(p in 1:(length(segLength)-1)){

TMP<-tmp

tt<-which(TMP[,c_len]>=segLength[p+1])
tt1<-table(TMP[,c_chr])

if(length(which(tt1==1))>0){tt1<-tt1[which(tt1==1)]
	tt1<-as.numeric(rownames(tt1))
	for(j in 1:length(tt1)){
		tt<-c(tt,which(TMP[,c_chr]==tt1[j]))
	}
tt<-as.numeric(rownames(table(tt)))
}

if(length(tt)>0){tMP<-TMP[-tt,];TMP<-TMP[tt,]}


TMP<-adjustTMPstableSubclone(TMP,recCHR)
#TMP<-getConfidence(TMP)

tmp[,c_conf]<-0

for(k in 1:dim(TMP)[[1]]){
	tt<-which(tmp[,c_is]>=TMP[k,c_is])
	if(length(tt)>0){kk<-which(tmp[tt,c_if]<=TMP[k,c_if])
	if(length(kk)>0){
		tmp[tt[kk],c_CN]<-TMP[k,c_CN]
		tmp[tt[kk],c_BA]<-TMP[k,c_BA]
		tmp[tt[kk],c_conf]<-k
	}}
}#k

for(k in 1:dim(tMP)[[1]]){
	tt<-which(tmp[,c_ind]==tMP[k,c_ind])
	if(length(tt)==1){
		tmp[tt,c_CN]<-tMP[k,c_CN]
		tmp[tt,c_BA]<-tMP[k,c_BA]
		tmp[tt,c_conf]<-(-k)
	}
}#k

}#p


out<-tmp
}#function


###########################################################
####
####   Function shortBreaksToTMP small segments are fitted, if possible, to exact neihbours
####
###########################################################

shortBreaksToTMP<-function(tmp){


recCHR<-CHR_table_recognition()

##### Enumerating and attributing short segments to big and stable ones


ss<-data.matrix(table(tmp[,c_chr],sign(tmp[,c_conf])))
tt<-as.numeric(which(ss[,2]==0))
if(length(tt)>0){
	ss<-as.numeric(rownames(ss))[tt]
	k<-1
	for(k in 1:length(ss)){
		kk<-which(tmp[,c_chr]==ss[k])
		dd<-max(tmp[1:kk[1],c_conf])
		for(p in 1:length(kk)){tmp[kk[p],c_conf]<-dd+p}
		if(kk[length(kk)]<dim(tmp)[[1]]){
		tmp[(kk[length(kk)]+1):dim(tmp)[[1]],c_conf]<-tmp[(kk[length(kk)]+1):dim(tmp)[[1]],c_conf]+length(kk)
		}
	}#k
}


tt<-matrix(0,2,2)
t_pos<-0
t_chr<-1
k_neg<-0

k<-64
for(k in 1:dim(tmp)[[1]]){
  if(tmp[k,c_chr]==t_chr){
	if(tmp[k,c_conf]<0){k_neg<-k_neg+1
	}else{if(k_neg>0){
		if(is.na(t_pos)){tt<-cbind(tt,c(tmp[k,c_conf],k_neg));t_pos<-tmp[k,c_conf]
		}else{if(tmp[k,c_conf]==t_pos){tt<-cbind(tt,c(t_pos,k_neg))
			}else{tt<-cbind(tt,c((t_pos+tmp[k,c_conf])/2,k_neg));t_pos<-tmp[k,c_conf]}
			}
		k_neg<-0
		}else{t_pos<-tmp[k,c_conf]}
		}
  }else{if(k_neg>0){tt<-cbind(tt,c(t_pos,k_neg));k_neg<-0}
		t_pos<-NA;t_chr<-tmp[k,c_chr]
		if(tmp[k,c_conf]<0){k_neg<-k_neg+1}else{t_pos<-tmp[k,c_conf]}
	}
}#k

Rest0<-0

if(dim(tt)[[2]]>4){tt<-tt[,-c(1,2)];Rest0<-2}else{tt<-cbind(tt[,-c(1,2)],tt[,-c(1,2)]);Rest0<-1}

if(Rest0>0){

tt<-data.matrix(tt)
colnames(tt)<-seq(1,dim(tt)[[2]])

kk<-which(tmp[,c_conf]>0)
if(length(kk)>0){tmpS<-tmp[-kk,]
if((dim(tmp)[[1]]-length(kk)==1)||(Rest0==1)){tmpS<-rbind(tmpS,tmpS)}
tmpB<-tmp[kk,]
}


tmpB[1,c_is]<-1

for(k in 2:dim(tmpB)[[1]]){
	if(tmpB[k-1,c_chr]!=tmpB[k,c_chr]){
		kk<-which(as.numeric(names(recCHR))==tmpB[k-1,c_chr])
		tmpB[k-1,c_if]<-recCHR[kk]
		tmpB[k,c_is]<-recCHR[kk]+1
	}
}

tmpB<-shrinkReprTMP(tmpB)

tt1<-which(tt[1,]==0.5)
if(length(tt1)>0){tt[1,tt1]<-1}

tmpS[,c_nS]<-1
tt_end<-length(which(tt[1,]<=dim(tmpB)[[1]]))

TT<-1
for(j in 1:tt_end){
	
  if(tt[2,j]>1){tt1<-TT:(TT+tt[2,j]-1)}else{tt1<-TT}

  tmpS[tt1,c_adH]<-tt[1,j]

	if(trunc(tt[1,j])==tt[1,j]){
		tmpS[tt1,c_CN]<-tmpB[tt[1,j],c_CN]
		tmpS[tt1,c_BA]<-tmpB[tt[1,j],c_BA]
	}
	TT<-TT+abs(tt[2,j])
}#j


####### Fit  short insertions
####### 1. Fit single supershort insertions within segment

tmpS[,c_nS]<-1
tt_end<-length(which(tt[1,]<=dim(tmpB)[[1]]))

TT<-1
j<-26
for(j in 1:tt_end){
	
  if(trunc(tt[1,j])==tt[1,j]){
   if(tt[2,j]==1){
	if(tmpS[TT,c_len]<15){tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])
	}else{
	  if(abs(tmpS[TT,c_nC]-tmpS[TT,c_CN])<1){
		if(tmpS[TT,c_len]<30){tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])
		}else{
			if(((tmpS[TT,c_CN]==tmpS[TT,c_CN1])&&((tmpS[TT,c_BA1]==0)||(tmpS[TT,c_BA1]==tmpS[TT,c_BA])||(tmpS[TT,c_BA]==0)))
				||((tmpS[TT,c_CN]==tmpS[TT,c_CN1+3])&&((tmpS[TT,c_BA1+3]==0)||(tmpS[TT,c_BA1+3]==tmpS[TT,c_BA])||(tmpS[TT,c_BA1+3]==0)))
				||((tmpS[TT,c_CN]==tmpS[TT,c_CN1+6])&&((tmpS[TT,c_BA1+6]==0)||(tmpS[TT,c_BA1+3]==tmpS[TT,c_BA])||(tmpS[TT,c_BA1+6]==0)))){
				tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])
			}else{}
			}
	  }else{if(min(tmpS[TT,c_nC],tmpS[TT,c_CN])>5){tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])}else{}

		}
		}
    }
  }
	TT<-TT+abs(tt[2,j])
}#j



Rest1<-0
kk<-which(tt[2,]<0)
if(length(kk)>0){
	if(dim(tt)[[2]]>length(kk)+1){tt<-tt[,-kk];Rest1<-2
	}else{if(dim(tt)[[2]]==length(kk)+1){
			if(Rest0==2){tt<-cbind(tt[,-kk],tt[,-kk]);Rest1<-1}
	}}
}

kk<-which(tmpS[,c_nS]==0)
if(length(kk)>0){
	if(dim(tmpS)[[1]]>length(kk)+1){tmpS<-tmpS[-kk,]
	}else{if(dim(tmpS)[[1]]==length(kk)+1){tmpS<-rbind(tmpS[-kk,],tmpS[-kk,])}
		}
}



####### 2. Fit single supershort insertions between segment
if(Rest1>0){

tt_end<-length(which(tt[1,]<=dim(tmpB)[[1]]))
if((Rest1==1)&&(tt_end==2)){tt_end<-1}

TT<-1
j<-26

for(j in 1:tt_end){
	
  if(trunc(tt[1,j])!=tt[1,j]){
	kk<-trunc(tt[1,j])
	if(tt[2,j]==1){
	  if(tmpS[TT,c_len]<15){
		if(!is.na(tmpS[TT,c_cn])){
			if(kk<dim(tmpB)[[1]]){
				pp<-which.min(c(abs(tmpS[TT,c_cn]-tmpB[kk,c_cn]),abs(tmpS[TT,c_cn]-tmpB[kk+1,c_cn])))
			}else{pp<-1}
			tmpS[TT,c_adH]<-(-(kk+pp-1));tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])

		}else{tmpS[TT,c_adH]<-(-kk);tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])}

	}else{
	  if(kk<dim(tmpB)[[1]]){

		if((abs(tmpS[TT,c_nC]-tmpB[kk,c_CN])<1)&&(abs(tmpS[TT,c_nC]-tmpB[kk+1,c_CN])>=1))
				{tmpS[TT,c_adH]<-(-kk);tmpS[TT,c_nS]<-(-1)}

		if((abs(tmpS[TT,c_nC]-tmpB[kk,c_CN])>=1)&&(abs(tmpS[TT,c_nC]-tmpB[kk+1,c_CN])<1))
				{tmpS[TT,c_adH]<-(-kk-1);tmpS[TT,c_nS]<-(-1)}

		if((abs(tmpS[TT,c_nC]-tmpB[kk,c_CN])<1)&&(abs(tmpS[TT,c_nC]-tmpB[kk+1,c_CN])<1))
				{pp<-which.min(c(tmpS[TT,c_cn],tmpS[TT,c_cn])-tmpB[kk:(kk+1),c_cn])
				tmpS[TT,c_adH]<-(-(kk+pp-1));tmpS[TT,c_nS]<-(-1)}

		if((tmpS[TT,c_adH]<0)&&(abs(tmpS[TT,c_adH])==trunc(abs(tmpS[TT,c_adH])))&&(tmpS[TT,c_len]<20)){
			tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])
			}


	  }else{if(abs(tmpS[TT,c_nC]-tmpB[kk,c_CN])<1){tmpS[TT,c_adH]<-(-kk);tmpS[TT,c_nS]<-0;tt[2,j]<-(-tt[2,j])}}
	}
	kk<-abs(tmpS[TT,c_adH])
	if((trunc(kk)==kk)&&(kk!=0)){tmpS[TT,c_CN]<-tmpB[kk,c_CN];tmpS[TT,c_BA]<-tmpB[kk,c_BA]}
  }
}

	TT<-TT+abs(tt[2,j])
}#j


if((dim(tmpS)[[1]]==2)&&(tmpS[1,c_ind]==tmpS[2,c_ind])){dim_tmpS<-1}else{dim_tmpS<-dim(tmpS)[[1]]}
k<-66
for(k in 1:dim_tmpS){
	if((tmpS[k,c_adH]<0)&&(tmpS[k,c_nS]==0)){
		tmpS[k,c_adH]<-abs(tmpS[k,c_adH])
		if(tmpS[k,c_is]==tmpB[tmpS[k,c_adH],c_if]+1){
			tmpB[tmpS[k,c_adH],c_if]<-tmpS[k,c_if]
			tmpB[tmpS[k,c_adH],c_ifCN]<-tmpS[k,c_ifCN]
			tmpB[tmpS[k,c_adH],c_len]<-tmpB[tmpS[k,c_adH],c_len]+tmpS[k,c_len]
		}else{
			if(tmpS[k,c_if]==tmpB[tmpS[k,c_adH],c_is]-1){
				tmpB[tmpS[k,c_adH],c_is]<-tmpS[k,c_is]
				tmpB[tmpS[k,c_adH],c_isCN]<-tmpS[k,c_isCN]
				tmpB[tmpS[k,c_adH],c_len]<-tmpB[tmpS[k,c_adH],c_len]+tmpS[k,c_len]
			}else{tmpS[k,c_nS]<-1;print("ZHOPA")}
			}
	}
}#k


Rest2<-0
kk<-which(tt[2,]<0)
if(length(kk)>0){
	if(dim(tt)[[2]]>length(kk)+1){tt<-tt[,-kk];Rest2<-2
	}else{if(dim(tt)[[2]]==length(kk)+1){
			if(Rest1==2){tt<-cbind(tt[,-kk],tt[,-kk]);Rest2<-1}
	}}
}else{Rest2<-Rest1}

kk<-which(tmpS[,c_nS]==0)
if(length(kk)>0){
	if((dim(tmpS)[[1]]==2)&&(tmpS[1,c_ind]==tmpS[2,c_ind])){Rest2<-0
	}else{if(dim(tmpS)[[1]]>length(kk)+1){tmpS<-tmpS[-kk,]
		}else{if(dim(tmpS)[[1]]==length(kk)+1){tmpS<-rbind(tmpS[-kk,],tmpS[-kk,])}
			}
		}
}

####### 3. Fit multiple insertions 
if(Rest2>0){

TT<-1
tt_end<-length(which(tt[1,]<=dim(tmpB)[[1]]))
if((Rest2==1)&&(tt_end==2)){tt_end<-1}

j<-1309

for(j in 1:tt_end){
	
  if(trunc(tt[1,j])==tt[1,j]){
	if(tt[2,j]>1){
		if((max(tmpS[TT:(TT+tt[2,j]-1),c_len])<15)&&(max(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpS[TT:(TT+tt[2,j]-1),c_CN]),na.rm=T)<1)){
			tmpS[TT:(TT+tt[2,j]-1),c_nS]<-0

		}else{tt1<-which(tmpS[TT:(TT+tt[2,j]-1),c_len]>=15)
 			if(length(tt1)>0){tt1<-TT+tt1-1

			if((max(abs(tmpS[tt1,c_nC]-tmpS[tt1,c_CN]))<1)&&(max(tmpS[tt1,c_len])<30)){
				tmpS[tt1,c_nS]<-0

			}else{p<-1
				for(p in 1:length(tt1)){
					if(tmpS[tt1[p],c_len]>=30){
						if(((tmpS[tt1[p],c_CN]==tmpS[tt1[p],c_CN1])&&((tmpS[tt1[p],c_BA1]==0)||(tmpS[tt1[p],c_BA1]==tmpS[tt1[p],c_BA])||(tmpS[tt1[p],c_BA]==0)))
					||((tmpS[tt1[p],c_CN]==tmpS[tt1[p],c_CN1+3])&&((tmpS[tt1[p],c_BA1+3]==0)||(tmpS[tt1[p],c_BA1+3]==tmpS[tt1[p],c_BA])||(tmpS[tt1[p],c_BA1+3]==0)))
					||((tmpS[tt1[p],c_CN]==tmpS[tt1[p],c_CN1+6])&&((tmpS[tt1[p],c_BA1+6]==0)||(tmpS[tt1[p],c_BA1+6]==tmpS[tt1[p],c_BA])||(tmpS[tt1[p],c_BA1+6]==0)))){
							tmpS[tt1[p],c_nS]<-0
						}
					}else{if((tmpS[tt1[p],c_CN]==tmpS[tt1[p],c_CN1])||(tmpS[tt1[p],c_CN]==tmpS[tt1[p],c_CN1+3])||(tmpS[tt1[p],c_CN]==tmpS[tt1[p],c_CN1+6])){
							tmpS[tt1[p],c_nS]<-0}
						}
				}#p
				}
			}else{tt1<-which(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpS[TT:(TT+tt[2,j]-1),c_CN])>=1)
				if(length(tt1)<=1){tmpS[TT:(TT+tt[2,j]-1),c_nS]<-0
}
				}
		}
	}
  }else{kk<-trunc(tt[1,j])
	if((tt[2,j]>1)&&(kk<dim(tmpB)[[1]])){

	   tt1<-which(tmpS[TT:(TT+tt[2,j]-1),c_len]>=15)

 	   if(length(tt1)==0){
		if((max(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[kk,c_CN]),na.rm=T)<1)
			&&(min(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[kk+1,c_CN]),na.rm=T)>=1)){
			tmpS[TT:(TT+tt[2,j]-1),c_adH]<-(-kk);tmpS[TT:(TT+tt[2,j]-1),c_nS]<-0
		}

		if((max(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[kk+1,c_CN]),na.rm=T)<1)
			&&(min(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpB[kk,c_CN]),na.rm=T)>=1)){
			tmpS[TT:(TT+tt[2,j]-1),c_adH]<-(-kk-1);tmpS[TT:(TT+tt[2,j]-1),c_nS]<-0
		}

	   }else{   tt1<-TT+tt1-1	
			if((max(abs(tmpS[tt1,c_nC]-tmpB[kk,c_CN]))<1)&&(min(abs(tmpS[tt1,c_nC]-tmpB[kk+1,c_CN]))>=1))
				{tmpS[tt1,c_adH]<-(-kk);tmpS[tt1,c_nS]<-0}

			if((min(abs(tmpS[tt1,c_nC]-tmpB[kk,c_CN]))>=1)&&(max(abs(tmpS[tt1,c_nC]-tmpB[kk+1,c_CN]))<1))
				{tmpS[tt1,c_adH]<-(-kk-1);tmpS[tt1,c_nS]<-0}

			if(sum(tmpS[tt1,c_nS])!=0){
				t_adH<-tmpS[tt1[1],c_adH]
					
				p<-0
				while(((p+1)<=length(tt1))&&(abs(tmpS[tt1[p+1],c_nC]-tmpB[kk,c_CN])<1)){
					tmpS[tt1[p+1],c_adH]<-(-kk);tmpS[tt1[p+1],c_nS]<-(-1);p<-p+1}
				if(p!=0){p_up<-seq(1,p)}else{p_up<-numeric(0)}

				p<-0
				while(((p+1)<=length(tt1))&&(abs(tmpS[tt1[length(tt1)-p],c_nC]-tmpB[kk+1,c_CN])<1)){
					tmpS[tt1[length(tt1)-p],c_adH]<-(-kk-1);tmpS[tt1[length(tt1)-p],c_nS]<-(-1);p<-p+1}
		
				if(p!=0){p_down<-seq(length(tt1)-p+1,length(tt1))}else{p_down<-numeric(0)}
					
				p_unkn<-intersect(p_up,p_down)
					
				if(length(p_unkn)>0){tmpS[tt1[p_unkn],c_adH]<-(t_adH);tmpS[tt1[p_unkn],c_nS]<-1
				}else{p_unkn<-setdiff(seq(1,length(tt1)),union(p_up,p_down))
					if(length(p_unkn)>0){tmpS[tt1[p_unkn],c_adH]<-(t_adH);tmpS[tt1[p_unkn],c_nS]<-1}
					}
			}	
		}

	}else{ if(tt[2,j]>1){tt1<-which(tmpS[TT:(TT+tt[2,j]-1),c_len]>=15)
			if(length(tt1)==0){
				if(max(abs(tmpS[TT:(TT+tt[2,j]-1),c_nC]-tmpS[kk,c_CN]),na.rm=T)<1){
					tmpS[TT:(TT+tt[2,j]-1),c_adH]<-(-kk);tmpS[TT:(TT+tt[2,j]-1),c_nS]<-0
				}
			}else{if(max(abs(tmpS[tt1,c_nC]-tmpB[kk,c_CN]))<1){tmpS[tt1,c_adH]<-(-kk);tmpS[tt1,c_nS]<-0}
				  }
		}
		}
	

  }#between segments
				
TT<-TT+abs(tt[2,j])
		
}#j


if((dim(tmpS)[[1]]==2)&&(tmpS[1,c_ind]==tmpS[2,c_ind])){dim_tmpS<-1}else{dim_tmpS<-dim(tmpS)[[1]]}

k<-154
for(k in 1:dim_tmpS){
	if((tmpS[k,c_adH]<0)&&(tmpS[k,c_nS]==0)){
		tmpS[k,c_adH]<-abs(tmpS[k,c_adH])
		if(tmpS[k,c_is]>=tmpB[tmpS[k,c_adH],c_if]+1){
		tmpB[tmpS[k,c_adH],c_if]<-tmpS[k,c_if]
		tmpB[tmpS[k,c_adH],c_ifCN]<-tmpS[k,c_ifCN]

		tmpB[tmpS[k,c_adH],c_len]<-tmpB[tmpS[k,c_adH],c_len]+tmpS[k,c_len]
		}else{
		if(tmpS[k,c_if]<=tmpB[tmpS[k,c_adH],c_is]-1){
		tmpB[tmpS[k,c_adH],c_is]<-tmpS[k,c_is]
		tmpB[tmpS[k,c_adH],c_isCN]<-tmpS[k,c_isCN]

		tmpB[tmpS[k,c_adH],c_len]<-tmpB[tmpS[k,c_adH],c_len]+tmpS[k,c_len]
		}else{
		if((tmpS[k,c_is]>=tmpB[tmpS[k,c_adH],c_is])&&(tmpS[k,c_if]<=tmpB[tmpS[k,c_adH],c_if])){
			tmpB[tmpS[k,c_adH],c_len]<-tmpB[tmpS[k,c_adH],c_len]+tmpS[k,c_len]
		}else{print("ZHOPA")}
		}
		}
	}
}#k


Rest3<-2

kk<-which(tmpS[,c_nS]==0)

if(length(kk)>0){
	if((dim(tmpS)[[1]]==2)&&(tmpS[1,c_ind]==tmpS[2,c_ind])){Rest3<-0
	}else{if(dim(tmpS)[[1]]>length(kk)+1){tmpS<-tmpS[-kk,]
		}else{if(dim(tmpS)[[1]]==length(kk)+1){tmpS<-rbind(tmpS[-kk,],tmpS[-kk,]);Rest3<-1}
			}
		}
}else{if((dim(tmpS)[[1]]==2)&&(tmpS[1,c_ind]==tmpS[2,c_ind])){Rest3<-1}}

#### Adjust tmpS with new copy numbers
if(Rest3>0){

tt<-which(tmpS[,c_nS]<0)
if(length(tt)>0){

tmpS[tt,c_CN]<-tmpB[abs(tmpS[tt,c_adH]),c_CN]
tmpS[tt,c_BA]<-tmpB[abs(tmpS[tt,c_adH]),c_BA]

for(p in 1:length(tt)){
if((tmpS[tt[p],c_CN1]==tmpS[tt[p],c_CN])&&((tmpS[tt[p],c_BA1]==tmpS[tt[p],c_BA])||(tmpS[tt[p],c_BA1]==0))){tmpS[tt[p],c_nS]<-0}
if((tmpS[tt[p],c_CN1+3]==tmpS[tt[p],c_CN])&&((tmpS[tt[p],c_BA1+3]==tmpS[tt[p],c_BA])||(tmpS[tt[p],c_BA1+3]==0))){tmpS[tt[p],c_nS]<-0}
if((tmpS[tt[p],c_CN1+6]==tmpS[tt[p],c_CN])&&((tmpS[tt[p],c_BA1+6]==tmpS[tt[p],c_BA])||(tmpS[tt[p],c_BA1+6]==0))){tmpS[tt[p],c_nS]<-0}
}


for(k in 1:dim(tmpS)[[1]]){
	if((tmpS[k,c_adH]<0)&&(tmpS[k,c_nS]==0)){
		tmpS[k,c_adH]<-abs(tmpS[k,c_adH])
		if(tmpS[k,c_is]>=tmpB[tmpS[k,c_adH],c_if]+1){
		tmpB[tmpS[k,c_adH],c_if]<-tmpS[k,c_if]
		tmpB[tmpS[k,c_adH],c_ifCN]<-tmpS[k,c_ifCN]
		tmpB[tmpS[k,c_adH],c_len]<-tmpB[tmpS[k,c_adH],c_len]+tmpS[k,c_len]
		}else{
		if(tmpS[k,c_if]<=tmpB[tmpS[k,c_adH],c_is]-1){
		tmpB[tmpS[k,c_adH],c_is]<-tmpS[k,c_is]
		tmpB[tmpS[k,c_adH],c_isCN]<-tmpS[k,c_isCN]
		tmpB[tmpS[k,c_adH],c_len]<-tmpB[tmpS[k,c_adH],c_len]+tmpS[k,c_len]
		}else{
		if((tmpS[k,c_is]>=tmpB[tmpS[k,c_adH],c_is])&&(tmpS[k,c_if]<=tmpB[tmpS[k,c_adH],c_if])){
			tmpB[tmpS[k,c_adH],c_len]<-tmpB[tmpS[k,c_adH],c_len]+tmpS[k,c_len]
		}else{print("ZHOPA")}

		}
		}
	}
}#k


kk<-which(tmpS[,c_nS]==0)
if(length(kk)>0){tmpS<-tmpS[-kk,]}


}#if tt>0


}# Rest3
}# Rest2
}# Rest1

#############################
### resulting tmp 
tmp[,c_conf]<-0

for(k in 1:dim(tmpS)[[1]]){
	kk<-which(tmp[,c_ind]==tmpS[k,c_ind])
	if(length(kk)==1){if(tmpS[k,c_conf]<0){tmp[kk,c_conf]<-tmpS[k,c_conf]}}
}



for(k in 1:dim(tmpB)[[1]]){
	tt<-which(tmp[,c_is]>=tmpB[k,c_is])
	if(length(tt)>0){kk<-which(tmp[tt,c_if]<=tmpB[k,c_if])
	if(length(kk)>0){
		pp<-which(tmp[tt[kk],c_conf]>=0)
		if(length(pp)>0){
			tmp[tt[kk[pp]],c_CN]<-tmpB[k,c_CN]
			tmp[tt[kk[pp]],c_BA]<-tmpB[k,c_BA]
			tmp[tt[kk[pp]],c_conf]<-tmpB[k,c_conf]
	}}}
}#k

}## if Rest0

out<-tmp

}#function




#################################################
############## Function assigns the number of region
#############
#################################################

getConfidence<-function(tmp){

tmp[,c_conf]<-0
tmp[1,c_conf]<-1

for(k in 2:dim(tmp)[[1]]){
	if((tmp[k,c_chr]==tmp[k-1,c_chr])&&(tmp[k,c_CN]==tmp[k-1,c_CN])&&(tmp[k,c_BA]==tmp[k-1,c_BA])){
		tmp[k,c_conf]<-tmp[k-1,c_conf]
	}else{tmp[k,c_conf]<-tmp[k-1,c_conf]+1}
}#k

out<-tmp

}# function




###########################################################
####
####   Function UniteSEGtmp_new Uniting segments with similar attribution
####
###########################################################

fitGermHomozyg<-function(tmp,Length){

FL<-T
while(FL==T){
FL<-F

tmp<-UniteSEG_TMP(tmp)

for(k in 1:dim(tmp)[[1]]){
	if((tmp[k,c_BA]==0)&&(tmp[k,c_len]>=1000)){tmp[k,c_BA]<-tmp[k,c_CN]}
}#k


for(k in 2:(dim(tmp)[[1]]-1)){

	if((tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k,c_BA]==0)
		&&(tmp[k-1,c_CN]==tmp[k+1,c_CN])&&(tmp[k-1,c_BA]==tmp[k+1,c_BA])&&(tmp[k+1,c_BA]!=0)
		&&(tmp[k,c_CN]==tmp[k+1,c_CN])&&(tmp[k,c_len]<=Length)){

		tmp[k,c_BA]<-tmp[k+1,c_BA]
		FL<-T
	}
}#k


for(k in 2:(dim(tmp)[[1]]-1)){

	if((tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k,c_BA]==tmp[k,c_CN])
		&&(tmp[k-1,c_CN]==tmp[k+1,c_CN])&&(tmp[k-1,c_BA]==tmp[k+1,c_BA])&&(tmp[k+1,c_BA]!=0)
		&&(tmp[k,c_CN]==tmp[k+1,c_CN])&&(tmp[k,c_len]<=Length)){

		tmp[k,c_BA]<-tmp[k+1,c_BA]
		FL<-T
	}
}#k


for(k in 2:(dim(tmp)[[1]]-1)){
	if(tmp[k,c_BA]==0){
		if((tmp[k,c_chr]==tmp[k+1,c_chr])
		&&(tmp[k,c_CN]==tmp[k+1,c_CN])&&(tmp[k+1,c_BA]!=0)&&(tmp[k,c_len]<=Length)
		&&((tmp[k-1,c_CN]!=tmp[k,c_CN])||(tmp[k-1,c_chr]!=tmp[k,c_chr]))){

		tmp[k,c_BA]<-tmp[k+1,c_BA]
		FL<-T
		}

		if((tmp[k,c_chr]==tmp[k-1,c_chr])
		&&(tmp[k,c_CN]==tmp[k-1,c_CN])&&(tmp[k-1,c_BA]!=0)&&(tmp[k,c_len]<=Length)
		&&((tmp[k+1,c_CN]!=tmp[k,c_CN])||(tmp[k+1,c_chr]!=tmp[k,c_chr]))){

		tmp[k,c_BA]<-tmp[k-1,c_BA]
		FL<-T
		}
}


}#k

} #FL

out<-tmp

}# function



###########################################################
####
####   Function checkSmallHomozygosities_FinalPolish 
####    smoothing out small segments reported as germline himozygosities 
####
###########################################################


checkSmallHomozygosities_FinalPolish<-function(tmp,Length){

tmp[,c_ind]<-seq(1,dim(tmp)[[1]])

TMP<-tmp

TMP<-fitGermHomozyg(TMP,Length)

for(k in 2:(dim(TMP)[[1]]-1)){
	if((TMP[k-1,c_chr]==TMP[k+1,c_chr])&&(max(TMP[k-1,c_len],TMP[k+1,c_len])/TMP[k,c_len]>3)&&(TMP[k,c_CN1+3]!=(-1))
		&&(TMP[k-1,c_CN]==TMP[k+1,c_CN])&&(TMP[k-1,c_BA]==TMP[k+1,c_BA])&&(TMP[k+1,c_BA]!=0)
		&&(TMP[k,c_CN]==TMP[k+1,c_CN])&&(TMP[k,c_BA]!=TMP[k+1,c_BA])&&(TMP[k,c_len]<=Length)
		){

		if((TMP[k,c_CN1]==TMP[k+1,c_CN])&&((TMP[k,c_BA1]==TMP[k+1,c_BA])||(TMP[k,c_BA1]==0))){TMP[k,c_BA]<-TMP[k,c_BA1]}
		if((TMP[k,c_CN1+3]==TMP[k+1,c_CN])&&((TMP[k,c_BA1+3]==TMP[k+1,c_BA])||(TMP[k,c_BA1+3]==0))){TMP[k,c_BA]<-TMP[k,c_BA1+3]}
		if((TMP[k,c_CN1+6]==TMP[k+1,c_CN])&&((TMP[k,c_BA1+6]==TMP[k+1,c_BA])||(TMP[k,c_BA1+6]==0))){TMP[k,c_BA]<-TMP[k,c_BA1+6]}
		
		tmp[TMP[k,c_ind],c_BA]<-TMP[k,c_BA]
		}
}#k


TMP<-fitGermHomozyg(TMP,Length)


tmp[,c_conf]<-0
for(k in 1:dim(TMP)[[1]]){
	tt<-which(tmp[,c_is]>=TMP[k,c_is])
	if(length(tt)>0){kk<-which(tmp[tt,c_if]<=TMP[k,c_if])
	if(length(kk)>0){
		tmp[tt[kk],c_CN]<-TMP[k,c_CN]
		tmp[tt[kk],c_BA]<-TMP[k,c_BA]
		tmp[tt[kk],c_conf]<-k
	}}
}#k


out<-tmp

}#function



###########################################################
####
####   Function checkHomozygosities_FinalPolish 
####    In the case of the nearly pure tumor sample it is hard to distinguish
####      germline and aquired homozygosities.
####    By default they are often attributed as germline, which is not true
####     So, function corrects this for almost pure tumors
####
###########################################################


checkHomozygosities_FinalPolish<-function(tmp,Length,p_BAF){

tt<-which(tmp[,c_BA]==0)
if(length(tt)>0){germHomo<-sum(tmp[tt,c_len])/sum(tmp[,c_len])}else{germHomo<-0}


for(k in 1:dim(tmp)[[1]]){
	if((tmp[k,c_BA]==0)&&(tmp[k,c_len]>=1000)){tmp[k,c_BA]<-tmp[k,c_CN]}
}#k


if((p_BAF<0.25)||(germHomo>0.1)){

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k,c_BA]==0)&&(tmp[k,c_len]>=Length)){tmp[k,c_BA]<-tmp[k,c_CN]}
}#k

}#if BAF

out<-tmp

}#function

###########################################################
####
####   Function checkSmallDiscrepancies looks for <100 SNPs segments which
####    differe less than 1 in c_cn, but 1 in c_CN 
####     and corrects them accordingly
####
###########################################################

checkHomozygBreaks_finalPolish<-function(tmp){


FL<-T
while(FL==T){
FL<-F
k<-100
for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k,c_CN]>0)&&(tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k-1,c_CN]==tmp[k+1,c_CN])&&(tmp[k-1,c_CN]!=tmp[k,c_CN])){
	if((tmp[k,c_BA]==0)||(tmp[k,c_CN]==tmp[k,c_BA])){
	if((abs(tmp[k,c_nC]-tmp[k+1,c_nC])<1)||(abs(tmp[k,c_nC]-tmp[k-1,c_nC])<1)){
	if(abs(tmp[k,c_CN]-tmp[k,c_nC])>abs(tmp[k,c_nC]-tmp[k-1,c_CN])){
		tmp[k,c_CN]<-tmp[k-1,c_CN]
		if(tmp[k,c_BA]!=0){tmp[k,c_BA]<-tmp[k,c_CN]}
		FL<-T
	}}
	}}
}#k

for(k in 2:(dim(tmp)[[1]]-1)){
  if((tmp[k,c_CN]>0)&&(tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k-1,c_CN]!=tmp[k+1,c_CN])){
	if((tmp[k,c_BA]==0)||(tmp[k,c_CN]==tmp[k,c_BA])){
	if((abs(tmp[k,c_nC]-tmp[k+1,c_nC])<0.5)&&(abs(tmp[k+1,c_nC]-tmp[k+1,c_CN])<0.5)){
	if(abs(tmp[k,c_CN]-tmp[k,c_nC])>abs(tmp[k,c_nC]-tmp[k+1,c_CN])){
		tmp[k,c_CN]<-tmp[k+1,c_CN]
		if(tmp[k,c_BA]!=0){tmp[k,c_BA]<-tmp[k,c_CN]}
		FL<-T

	}}
	if((abs(tmp[k,c_nC]-tmp[k-1,c_nC])<0.5)&&(abs(tmp[k-1,c_nC]-tmp[k-1,c_CN])<0.5)){
	if(abs(tmp[k,c_CN]-tmp[k,c_nC])>abs(tmp[k,c_nC]-tmp[k-1,c_CN])){
		tmp[k,c_CN]<-tmp[k-1,c_CN]
		if(tmp[k,c_BA]!=0){tmp[k,c_BA]<-tmp[k,c_CN]}
		FL<-T

	}}
  }}
}#k

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k+1,c_chr]!=tmp[k,c_chr])&&(tmp[k,c_CN]>0)&&((tmp[k,c_BA]==0)||(tmp[k,c_CN]==tmp[k,c_BA]))){
	if((tmp[k-1,c_chr]==tmp[k,c_chr])&&(tmp[k,c_CN]!=tmp[k-1,c_CN])&&(abs(tmp[k,c_nC]-tmp[k-1,c_nC])<1)&&(abs(tmp[k,c_CN]-tmp[k,c_nC])>abs(tmp[k,c_nC]-tmp[k-1,c_CN]))){
		tmp[k,c_CN]<-tmp[k-1,c_CN]
		if(tmp[k,c_BA]!=0){tmp[k,c_BA]<-tmp[k,c_CN]}
		FL<-T
	}}
}#k

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k-1,c_chr]!=tmp[k,c_chr])&&(tmp[k,c_CN]>0)&&((tmp[k,c_BA]==0)||(tmp[k,c_CN]==tmp[k,c_BA]))){
	if((tmp[k+1,c_chr]==tmp[k,c_chr])&&(tmp[k,c_CN]!=tmp[k+1,c_CN])&&(abs(tmp[k,c_nC]-tmp[k+1,c_nC])<1)&&(abs(tmp[k,c_CN]-tmp[k,c_nC])>abs(tmp[k,c_nC]-tmp[k-1,c_CN]))){
		tmp[k,c_CN]<-tmp[k+1,c_CN]
		if(tmp[k,c_BA]!=0){tmp[k,c_BA]<-tmp[k,c_CN]}
		FL<-T
	}}
}#k

}#FL

out<-tmp

}#function




###########################################################
####
####   Function checkSmallDiscrepancies looks for <100 SNPs segments which
####    differe less than 1 in c_cn, but 1 in c_CN 
####     and corrects them accordingly
####
###########################################################

checkSmallDiscrepancies_finalPolish<-function(tmp,Length){

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k,c_len]<Length)&&(tmp[k,c_CN]>0)&&(max(tmp[k+1,c_len],tmp[k-1,c_len])>=2*Length)){
	if((tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k-1,c_CN]==tmp[k+1,c_CN])&&(tmp[k-1,c_CN]!=tmp[k,c_CN])){
	if((abs(tmp[k,c_nC]-tmp[k+1,c_nC])<1)||(abs(tmp[k,c_nC]-tmp[k-1,c_nC])<1)){
		tmp[k,c_CN]<-tmp[k-1,c_CN]
		tmp[k,c_BA]<-tmp[k-1,c_BA]

	}}
	}
}#k

for(k in 2:(dim(tmp)[[1]]-1)){
  if((tmp[k,c_len]<Length)&&(tmp[k,c_CN]>0)&&(tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k-1,c_CN]!=tmp[k+1,c_CN])){
	if((tmp[k+1,c_len]>=2*Length)&&(abs(tmp[k,c_nC]-tmp[k+1,c_nC])<=1)&&(abs(tmp[k,c_nC]-tmp[k-1,c_nC])>1)){
		tmp[k,c_CN]<-tmp[k+1,c_CN]
		tmp[k,c_BA]<-tmp[k+1,c_BA]

	}
	if((tmp[k-1,c_len]>=2*Length)&&(abs(tmp[k,c_nC]-tmp[k+1,c_nC])>1)&&(abs(tmp[k,c_nC]-tmp[k-1,c_nC])<=1)){
		tmp[k,c_CN]<-tmp[k-1,c_CN]
		tmp[k,c_BA]<-tmp[k-1,c_BA]

	}

  }
}#k

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k+1,c_chr]!=tmp[k,c_chr])&&(tmp[k,c_len]<Length)&&(tmp[k,c_CN]>0)){
	if((tmp[k-1,c_chr]==tmp[k,c_chr])&&(tmp[k-1,c_len]>=2*Length)&&(abs(tmp[k,c_nC]-tmp[k-1,c_nC])<1)){
		tmp[k,c_CN]<-tmp[k-1,c_CN]
		tmp[k,c_BA]<-tmp[k-1,c_BA]

	}}
}#k

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k-1,c_chr]!=tmp[k,c_chr])&&(tmp[k,c_len]<Length)&&(tmp[k,c_CN]>0)){
	if((tmp[k+1,c_chr]==tmp[k,c_chr])&&(tmp[k+1,c_len]>=2*Length)&&(abs(tmp[k,c_nC]-tmp[k+1,c_nC])<1)){
		tmp[k,c_CN]<-tmp[k+1,c_CN]
		tmp[k,c_BA]<-tmp[k+1,c_BA]

	}}
}#k


out<-tmp

}#function



###########################################################
####
####   Function iliminateShort_FinalPolish Uniting segments <15 SNPs 
####
###########################################################

iliminateShort_FinalPolish<-function(tmp){

for(k in 2:(dim(tmp)[[1]]-1)){

	if((tmp[k,c_len]<15)&&(tmp[k,c_nC]>0.5)||(tmp[k,c_len]<5)){

	if((tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k,c_conf]==tmp[k-1,c_conf])&&(tmp[k,c_conf]==tmp[k+1,c_conf])){
	if((is.na(tmp[k,c_cn]))||(abs(tmp[k,c_cn]-tmp[k+1,c_cn])>abs(tmp[k,c_cn]-tmp[k-1,c_cn]))){
		tmp[k-1,c_if]<-tmp[k,c_if]
		tmp[k-1,c_ifCN]<-tmp[k,c_ifCN]
		tmp[k-1,c_len]<-tmp[k-1,c_len]+tmp[k,c_len]
		tmp[k,]<-tmp[k-1,]
		tmp[k-1,1]<-0
	}else{if(abs(tmp[k,c_cn]-tmp[k+1,c_cn])<=abs(tmp[k,c_cn]-tmp[k-1,c_cn])){
		tmp[k+1,c_is]<-tmp[k,c_is]
		tmp[k+1,c_isCN]<-tmp[k,c_isCN]
		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k,1]<-0
		}}
	}
}
}#k
tt<-which(tmp[,1]==0);if(length(tt)!=0){tmp<-tmp[-tt,]}

for(k in 2:dim(tmp)[[1]]){
	if((tmp[k,c_len]<15)&&(tmp[k,c_nC]>0.5)||(tmp[k,c_len]<5)){
	if((tmp[k-1,c_chr]==tmp[k,c_chr])&&(tmp[k,c_conf]==tmp[k-1,c_conf])){
		tmp[k-1,c_if]<-tmp[k,c_if]
		tmp[k-1,c_ifCN]<-tmp[k,c_ifCN]
		tmp[k-1,c_len]<-tmp[k-1,c_len]+tmp[k,c_len]
		tmp[k,]<-tmp[k-1,]
		tmp[k-1,1]<-0
	}}
}#k
tt<-which(tmp[,1]==0);if(length(tt)!=0){tmp<-tmp[-tt,]}

for(k in 2:(dim(tmp)[[1]]-1)){
	if((tmp[k,c_len]<15)&&(tmp[k,c_nC]>0.5)||(tmp[k,c_len]<5)){
	if((tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k,c_conf]==tmp[k+1,c_conf])){
		tmp[k+1,c_is]<-tmp[k,c_is]
		tmp[k+1,c_isCN]<-tmp[k,c_isCN]
		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k,1]<-0
	}}
}#k
tt<-which(tmp[,1]==0);if(length(tt)!=0){tmp<-tmp[-tt,]}


for(k in 2:(dim(tmp)[[1]]-1)){

	if((tmp[k,c_len]<15)&&(tmp[k,c_nC]>0.5)||(tmp[k,c_len]<5)){
	if((tmp[k-1,c_chr]==tmp[k+1,c_chr])&&(tmp[k,c_conf]!=tmp[k-1,c_conf])&&(tmp[k,c_conf]!=tmp[k+1,c_conf])){
	if((is.na(tmp[k,c_cn]))||(abs(tmp[k,c_cn]-tmp[k+1,c_cn])>abs(tmp[k,c_cn]-tmp[k-1,c_cn]))){
		tmp[k-1,c_if]<-tmp[k,c_if]
		tmp[k-1,c_ifCN]<-tmp[k,c_ifCN]
		tmp[k-1,c_len]<-tmp[k-1,c_len]+tmp[k,c_len]
		tmp[k,]<-tmp[k-1,]
		tmp[k-1,1]<-0
	}else{if(abs(tmp[k,c_cn]-tmp[k+1,c_cn])<=abs(tmp[k,c_cn]-tmp[k-1,c_cn])){
		tmp[k+1,c_is]<-tmp[k,c_is]
		tmp[k+1,c_isCN]<-tmp[k,c_isCN]
		tmp[k+1,c_len]<-tmp[k+1,c_len]+tmp[k,c_len]
		tmp[k,1]<-0
		}}
	}
}
}#k
tt<-which(tmp[,1]==0);if(length(tt)!=0){tmp<-tmp[-tt,]}

tmp[,c_ind]<-seq(1,dim(tmp)[[1]])

tmp<-round(tmp,4)
tmp[,c_nC]<-round(tmp[,c_nC],1)

out<-tmp

}#function


###########################################################
####
####   Function updateMeans_FinalPolish
####
###########################################################

updateMeans_FinalPolish<-function(tmp,datA,datC,Centr_CL,p_BAF){

tmp[,c_ind]<-seq(1,dim(tmp)[[1]])

c_CL<-cbind(Centr_CL,Centr_CL[,which(Centr_CL[1,]-Centr_CL[2,]==0)])
c_CL[2,26:34]<-0
c_CL[3,26:34]<-c_CL[1,26:34]+30
c_CL[4,26:34]<-c_CL[5,26:34]+1
c_CL<-c_CL[,order(c_CL[3,],decreasing=F)]


Break<-(Centr_CL[5,21]-Centr_CL[5,20])/4

tt<-which(is.na(datA));if(length(tt)!=0){datA[tt]<-0}
tt<-which(is.na(datC));if(length(tt)!=0){datA[tt]<-0}

tmp[,c_nS]<-tmp[,c_cn]
tmp[,c_adH]<-tmp[,c_adL]

tt<-which(is.na(tmp[,c_adH]))
if(length(tt)>0){tmp[tt,c_adH]<-0}


TMP<-shrinkReprTMP(tmp)


for(k in 1:dim(TMP)[[1]]){
	if(TMP[k,c_ifCN]>TMP[k,c_isCN]){
	TMP[k,c_nS]<-median(datC[TMP[k,c_isCN]:TMP[k,c_ifCN]],na.rm=T)
	}else{TMP[k,c_nS]<-datC[TMP[k,c_isCN]]}	
}#k


TMP<-round(TMP,4)

for(k in 1:dim(TMP)[[1]]){

   if(TMP[k,c_len]>200){
	
	cl<-which(abs(c_CL[1,]-TMP[k,c_CN])==0)
	if(abs(TMP[k,c_cn]-c_CL[5,cl[1]])<=Break){TMP[k,c_nS]<-c_CL[5,cl[1]]}

	cn<-c_CL[,cl]
	
      ba<-which(abs(cn[2,]-TMP[k,c_BA])==0)
	ba<-ba[1]

	if(abs(TMP[k,c_adH]-cn[4,ba])>Break*2){

		adv<-datA[TMP[k,c_is]:TMP[k,c_if]]

		if(ba==1){KN<-0;KK<-(cn[4,ba+1]+cn[4,ba])/2
			}else{if(ba==dim(cn)[[2]]){KN<-(cn[4,ba-1]+cn[4,ba])/2;KK<-10
				}else{KN<-(cn[4,ba-1]+cn[4,ba])/2
					KK<-(cn[4,ba+1]+cn[4,ba])/2
					}
				}
		tt<-which(abs(adv)>=KN)
		if(length(tt)>0){tt1<-which(abs(adv[tt])<=KK)
			if(length(tt1)>0){tt<-tt[tt1]}
		}

		if(length(tt)>30){
			dd<-dip(abs(adv[tt]), full.result = T, debug = FALSE)
			TMP[k,c_adH]<-(dd$xl+dd$xu)/2
			if(abs(TMP[k,c_adH]-cn[4,ba])<=Break*2){TMP[k,c_adH]<-cn[4,ba]}
		}
	}else{TMP[k,c_adH]<-cn[4,ba]}	
}
}#k

TMP<-round(TMP,4)



tmp[,c_conf]<-0
for(k in 1:dim(TMP)[[1]]){
	tt<-which(tmp[,c_is]>=TMP[k,c_is])
	if(length(tt)>0){kk<-which(tmp[tt,c_if]<=TMP[k,c_if])
	if(length(kk)>0){
		tmp[tt[kk],c_nS]<-TMP[k,c_nS]
		tmp[tt[kk],c_adH]<-TMP[k,c_adH]
		tmp[tt[kk],c_conf]<-k
	}}
}#k

out<-tmp

}#function
